PSO.info.potentials = {}

rawset(_G, "POTTYPE_ONLYACTIVE", 0)	
rawset(_G, "POTTYPE_ONLYPASSIVE", 1)
rawset(_G, "POTTYPE_PASSIVE", 2)
rawset(_G, "POTTYPE_ANYPALETTE", 3)

-- registers a weapon's potential hook.
-- this adds the necessary checks depending on whether we want it to be passive, active or not.
-- num denotes the position of the player_t argument which we need to know to compare potential
-- num2 (if set) denotes the position of the mobj_t argument of a hitbox/missile that has a weapon saved to itself.
-- this is used to know if the potential saved in the missile and that in the player's currently equipped weapon match.
PSO.lib.registerPotentialHook = function(potential, hookn, type, fn, num, num2)
		
	local finalfunc = function(...)
		
		local args = {...}
		local p	-- try to find which one is a player here.
		if num or num == nil
			p = args[num or 1]
		end
		
		if not p
		or type == POTTYPE_ONLYACTIVE and PSO.lib.activeWeaponPotential(p, potential)
		or type == POTTYPE_ONLYPASSIVE and PSO.lib.passiveWeaponPotential(p, potential)
		or type == POTTYPE_PASSIVE and PSO.lib.activeOrPassivePotential(p, potential)
		or type == POTTYPE_ANYPALETTE and PSO.lib.paletteWeaponPotential(p, potential)
			
			-- projectile check...
			local proj
			if num2
				proj = args[num2]
				--print(proj)
				if proj and proj.valid and proj.pso and proj.pso.savewpn
					local info = PSO.lib.itemInfo(proj.pso.savewpn)
					if info.potential != potential	-- proj was fired with a weapon that does not match this one
						--print("potential nulled due to switching.")
						return	-- do nothing.
					end
				end
			end
			
			return fn(unpack(args))
		end
	end
	
	--print("Adding hook for potential "..potential)
	PSO.lib.addHook(hookn, finalfunc)
end

PSO.lib.registerPotential = function(potential)
	local pot = PSO.info.potentials[potential]
	if pot.funcs
		for i = 1, #pot.funcs
			local fndata = pot.funcs[i]
			if fndata.hookname and fndata.func
				PSO.lib.registerPotentialHook(potential, fndata.hookname, fndata.type or POTTYPE_PASSIVE, fndata.func, fndata.playerargindex, fndata.inflictorindex)
			end
		end
	end
end

-- 1~3*
PSO.info.potentials["state of the art"] = {
	
	name = "State of the art",
	desc = "Lowers PP cost of all attacks by 2 points, to a minimum of 1.",
	
	funcs = {
		
		{
			hookname = "PPCost",			-- hook to register into
			type = POTTYPE_ONLYACTIVE,		-- scope, when should this run?
			playerargindex = 1,				-- tells us which argument of the function the player is. (1 by default)
			func = 	function(p, cost)		-- function to run akin to the hook.
						if cost[1] > 0 
							cost[1] = max(1, $-2)
						end
					end,
		},
	}
}

-- 4~6*
PSO.info.potentials["photonic keeper"] = {
	
	name = "Photonic Keeper",
	desc = "Recover 10% of max PP every 10 seconds if HP is full. Effect timer starts when switched to and resets if HP falls below full.",
	
	funcs = {
		{
			hookname = "playerThink",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p)
						if p.pso.hp >= p.pso.maxhp 
							
							if (p.pso.pottimer%(10*TICRATE)) > TICRATE*5
								-- show that it's charging!
								local t2 = P_SpawnMobj(p.mo.x + P_RandomRange(-64, 64)*p.mo.scale, p.mo.y + P_RandomRange(-64, 64)*p.mo.scale, p.mo.z + P_RandomRange(-64, 64)*p.mo.scale, MT_THOK)
								t2.scale = p.mo.scale/4
								t2.destscale = p.mo.scale/8
								t2.scalespeed = p.mo.scale/12
								t2.frame = A
								t2.color = SKINCOLOR_CYAN
								
								t2.momx = (p.mo.x - t2.x)/10
								t2.momy = (p.mo.y - t2.y)/10
								t2.momz = (p.mo.z - t2.z)/10
								
								t2.fuse = 10
								t2.tics = 10								
							elseif p.pso.pottimer and p.pso.pottimer%(10*TICRATE) == 0
							
								PSO.lib.healPP(p.mo, p.mo, p.pso.maxpp/10)
							end
						else
							p.pso.pottimer = 0
						end
					end,
		},
	}
}

-- special low rarity gear:
-- Azul
PSO.info.potentials["lifespring"] = {
	
	name = "Lifespring",
	desc = "On Critical Hit, recover 10 HP once per attack.",
	
	funcs = {
			
			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer)
					
					if timer == 1
						p.mo.lifespringok = true
					end
				end,
			},			
			
			{
				hookname = "playerDamage",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg, skipchain, crit)
				
					if crit and p.mo.lifespringok
						PSO.lib.healObject(p.mo, p.mo, 10, true)
						p.mo.lifespringok = nil
					end
				end,
			},
	}
}

-- Exiled
PSO.info.potentials["firm stance"] = {
	
	name = "Firm Stance",
	desc = "Prevents knockback from attacks if HP is more than half.",
	
	funcs = {

			{
				hookname = "playerReaction",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				inflictorindex = 3,
				func = 	function(p, mo, inflictor, arg)
				
					if p.pso.hp > p.pso.maxhp/2
						arg[1] = RT_NONE
					end
				end,
			},
	},
}

-- Boreal
PSO.info.potentials["soulspring"] = {
	
	name = "Soulspring",
	desc = "On Critical Hit, recover 4 PP once per attack.",
	
	funcs = {

			
			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer)
					
					if timer == 1
						p.mo.soulspringok = true
					end
				end,
			},	

			{
				hookname = "playerDamage",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg, skipchain, crit)
				
					if crit and p.mo.soulspringok
						PSO.lib.healPP(p.mo, p.mo, 4, true)
						p.mo.soulspringok = nil
					end
				end,
			},
	}
}

-- Fusiel (combines the first 3 together!)
PSO.info.potentials["serene spirit"] = {
	
	name = "Serene Spirit",
	desc = "Prevents knockback when HP is more than half. On Critical Hit, recover 10 HP and 4 PP once per attack.",
	
	funcs = {
		PSO.info.potentials["lifespring"].funcs[1],
		PSO.info.potentials["lifespring"].funcs[2],
		PSO.info.potentials["soulspring"].funcs[1],
		PSO.info.potentials["soulspring"].funcs[2],
		PSO.info.potentials["firm stance"].funcs[1],
	},
}

PSO.info.potentials["hopeless manoeuver"] = {
	
	name = "Hopeless Manoeuver",
	desc = "On succesful dodge roll, increase Critical Hit Rate by 20% for 60s. Effect is lost when switching weapons.",

	funcs = {
	
		{
			hookname = "playerDodge",
			type = POTTYPE_PASSIVE,	-- also works for 1h weapons...
			playerargindex = 1,
			func = 	function(p)
						S_StartSoundAtVolume(p.mo, sfx_cburst, 127)
									
						for i = 1, 8
							local s = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_SUPERSPARK)
							s.destscale = 1
							s.momx = P_RandomRange(-32, 32)*FRACUNIT
							s.momy = P_RandomRange(-32, 32)*FRACUNIT
							s.momz = P_RandomRange(-32, 32)*FRACUNIT
							s.colorized = true
							s.color = SKINCOLOR_YELLOW
						end
						
						p.mo.resurgent_crit = TICRATE*60
					end,
		},
		
		{
			hookname = "playerThink",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						if p.mo.resurgent_crit
							p.mo.resurgent_crit = $-1
							
							if leveltime %2
								for i = 1, P_RandomRange(1, 4)
									local a = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z +p.mo.height/4, MT_THOK)
									a.sprite = SPR_KLIT
									a.colorized = true
									a.color = SKINCOLOR_GOLD
									a.scale = p.mo.scale/6
									a.tics = 4
									a.frame = P_RandomRange(0, P) |FF_FULLBRIGHT|TR_TRANS40|FF_ADD
									a.momx, a.momy, a.momz = p.mo.momx, p.mo.momy, p.mo.momz
								end								
							end
						end
					end,
		},		
		
		{
			hookname = "playerCritRate",
			type = POTTYPE_PASSIVE,	-- also works for 1h weapons...
			playerargindex = 1,
			func = 	function(p, mo, inflictor, arg)
						if p.mo.resurgent_crit
							arg[1] = $+20
						end
					end,
		},

		{
			hookname = "switchedWeapons",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						p.mo.resurgent_crit = nil
					end,
		},
	},
}

PSO.info.potentials["prototype unit ver.1"] = {
	
	name = "Prototype Unit Ver.1",
	desc = "Actions that would normally break Chain build it.",
	
	funcs = {
		
		{
			hookname = "playerAction",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p, timer)
				if timer == 1	-- only after starting an action.
					p.mo.alternate_chain_mode = true	-- this is a hack.
				end
			end,
		},

		{
			hookname = "switchingWeapons",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
				p.mo.alternate_chain_mode = nil	-- this is a hack.
			end,
		},
	},
}

PSO.info.potentials["prototype unit ver.2"] = {
	
	name = "Prototype Unit Ver.2",
	desc = "Photon Arts costs are deducted from your HP instead of your PP. You may be incapacitated by attempting to use a Photon Art.",
	
	funcs = {
			{
				hookname = "PPCost",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p, cost)
					
					if cost[1]
						PSO.lib.damageObject(p.mo, nil, cost[1])
						cost[1] = 0
					end
				end,
			},
	},
}

PSO.info.potentials["prototype unit ver.3"] = {
	
	name = "Prototype Unit Ver.3",
	desc = "When hitting a target, 30% chance to recover 5PP and 5% chance to lose 5% HP.",
	
	funcs = {

			{
				hookname = "playerDamage",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg, skipchain, crit)
				
					if P_RandomRange(0, 99) < 30
						PSO.lib.healPP(p.mo, p.mo, 5)
					end
					
					if P_RandomRange(0, 99) < 5
						PSO.lib.damageObject(p.mo, nil, p.pso.maxhp/20)
					end
				end,
			},
	},
}

PSO.info.potentials["prototype modchip"] = {
	
	name = "Prototype Modchip",
	desc = "Dodge roll swaps between 2 modes: Builds chain with any action and Photon Arts cost HP instead of PP, or 30% chance to recover 5 PP and 5% chance to lose 5% HP on hit.",
	
	funcs = {

		{
			hookname = "switchedWeapons",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
				p.mo.alternate_chain_mode = false
				p.mo.modchip_switch = 0
				p.mo.modchip_mode = 0
			end,
		},
		
		-- modchip switch.
		{
			hookname = "playerThink",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						if p.pso.dodgeroll
							
							if not p.mo.modchip_switch
								p.mo.modchip_mode = (not $) and 1 or 0
								p.mo.modchip_switch = true
								
								S_StartSound(p.mo, sfx_gravch)
																
								local g = P_SpawnGhostMobj(p.mo)
								g.scale = p.mo.scale
								g.colorized = true
								g.color = (p.mo.modchip_mode and SKINCOLOR_RED or SKINCOLOR_BLUE)
								g.destscale = p.mo.scale*8
								g.scalespeed = p.mo.scale/3
								p.mo.alternate_chain_mode = (not p.mo.modchip_mode)
							end
						else
							p.mo.modchip_switch = nil
						end

						for i = 1, 2
							if p.pso.weaponpos[i]
								local wep = p.pso.weaponpos[i][1]
								if wep and wep.valid
									wep.colorized = true
									wep.color = p.mo.modchip_mode and SKINCOLOR_RED or SKINCOLOR_BLUE
								end
							end
						end
						
					end,
		},
		
		
		-- now the other effects~
		-- mode 0
		{
			hookname = "PPCost",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p, cost)
				
				if not p.mo.modchip_mode
				and cost[1]
					PSO.lib.damageObject(p.mo, nil, cost[1])
					cost[1] = 0
				end
			end,
		},	
		
		-- mode 1
		{
			hookname = "playerDamage",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, mo, inflictor, dmg, skipchain, crit)
				
				if p.mo.modchip_mode
					if P_RandomRange(0, 99) < 30
						PSO.lib.healPP(p.mo, p.mo, 5)
					end
						
					if P_RandomRange(0, 99) < 5
						PSO.lib.damageObject(p.mo, nil, p.pso.maxhp/20)
					end
				end
			end,
		},		
	
		-- reset
		{
			hookname = "switchingWeapons",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
				p.mo.alternate_chain_mode = nil	-- this is a hack.
			end,
		},		
	
	},
}

PSO.info.potentials["tame prototype"] = {
	
	name = "Tame Prototype",
	desc = "Eliminates damage variance, always dealing 95% of damage. Enables synthesis of weapon upgrades.",
	
	funcs = {

			{
				hookname = "playerVariance",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, minvar, maxvar)
					minvar[1] = 95
					maxvar[1] = 95
				end,
			},
	},
}

PSO.info.potentials["aberration slayer"] = {
	
	name = "Aberration Slayer",
	desc = "When hitting an Aberration, recover 2 HP and 1 PP.",
	
	funcs = {

			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer)
					
					if timer == 1
						p.mo.gyrasok = true
					end
				end,
			},	

			{
				hookname = "playerDamage",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg, skipchain, crit)
				
					if mo.pso and mo.pso.attribute == ATTR_ABERRATION and p.mo.gyrasok
						PSO.lib.healObject(p.mo, p.mo, 5, true)
						PSO.lib.healPP(p.mo, p.mo, 2, true)
						p.mo.gyrasok = nil
					end
				end,
			},
	
	},
}

-- build potential by merging the other two.
local pot_glimmer = {}

for i = 1, #PSO.info.potentials["hopeless manoeuver"].funcs
	pot_glimmer[#pot_glimmer+1] = PSO.info.potentials["hopeless manoeuver"].funcs[i]
end
pot_glimmer[#pot_glimmer+1] = PSO.info.potentials["aberration slayer"].funcs[1]
pot_glimmer[#pot_glimmer+1] = PSO.info.potentials["aberration slayer"].funcs[2]

PSO.info.potentials["glimmer of hope"] = {
	
	name = "Glimmer of Hope",
	desc = "On succesful dodge roll, Critical Hit Rate +20% for 60s. Recover 5 HP and 2 PP when hitting Aberrations.",
	
	funcs = pot_glimmer
}

-- add the hopeless manoeuver part of the pot
local pot_spark = {}
for i = 1, #PSO.info.potentials["glimmer of hope"].funcs -2
	pot_spark[#pot_spark+1] = PSO.info.potentials["glimmer of hope"].funcs[i]
end
pot_spark[#pot_spark+1] = PSO.info.potentials["firm stance"].funcs[1]
pot_spark[#pot_spark+1] = 			{
			hookname = "playerAction",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, timer)
					
				if timer == 1
					p.mo.lumielok = true
				end
			end,
		}	
pot_spark[#pot_spark+1] = 			{
			hookname = "playerDamage",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, mo, inflictor, dmg, skipchain, crit)
				
				if p.mo.lumielok
					if crit
						PSO.lib.healObject(p.mo, p.mo, 10, true)
						PSO.lib.healPP(p.mo, p.mo, 4, true)
						p.mo.lumielok = nil
					elseif mo.pso and mo.pso.attribute == ATTR_ABERRATION		
						PSO.lib.healObject(p.mo, p.mo, 5, true)
						PSO.lib.healPP(p.mo, p.mo, 2, true)
						p.mo.lumielok = nil
					end
					p.mo.lumielok = nil
				end
			end,
		}

-- leave out aberration slayer, we will have to redo these.

PSO.info.potentials["spark of hope"] = {
	
	name = "Spark of Hope",
	desc = "On succesful dodge roll, Critical Hit Rate +20% for 60s. Recover HP and PP when landing Critical Hits or hitting Aberrations once per attack. Knockback from damage taken is negated if HP is above half.",
	funcs = pot_spark,
}

--------------------------------------------------------
-- 7* GEAR POTENTIALS --
--------------------------------------------------------

-- Orakl
PSO.info.potentials["perfect rhythm"] = {
	
	name = "Perfect Rhythm",
	desc = "Creates a timing window after letting an attack finish. Timing another attack as the circle becomes red restores 10% max PP.",
	
	funcs = {

			{
				hookname = "playerActionFinish",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p)
							p.pso.oraklja = TICRATE
				end,
			},
			
			{
				hookname = "playerThink",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p)
							if p.pso.oraklja

								local timer = p.pso.oraklja-(TICRATE/2)
								local scale = max(p.mo.scale*2,  p.mo.scale*2 + timer*(p.mo.scale))

								local ja = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_THOK)
								ja.sprite = SPR_ECRC
								ja.frame = C|FF_FULLBRIGHT|TR_TRANS40
								ja.tics = 1
								ja.scale = scale
								ja.color = p.pso.oraklja < TICRATE/2 and SKINCOLOR_RED or SKINCOLOR_BLUE
								
								if p ~= displayplayer
									ja.sprite = SPR_NULL
									ja.frame = A
								end
								
								p.pso.oraklja = $-1
							end
						end,
			},
			
			{
				hookname = "playerAction",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p)
							if p.pso.oraklja
								
								if p.pso.oraklja <= TICRATE/2
									PSO.lib.healPP(p.mo, p.mo, p.pso.maxpp*10/100)
								end
								
								p.pso.oraklja = nil
							end
				end,
			},

			{
				hookname = "switchingWeapons",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p)
								
							p.pso.oraklja = nil
				end,
			},

			{
				hookname = "playerReset",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p)
								
							p.pso.oraklja = nil
				end,
			},
			
		},
}

-- Vulture Basher
PSO.info.potentials["steel wing"] = {
	
	name = "Steel Wing",
	desc = "Increases the attack speed of Extend Cutter by 40%.",
	
	funcs = {
	
			{
				hookname = "attackSpeed",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, spd)
							if p.pso.action == "sword_pa_extendcutter"
								spd[1] = $+40
							end
				end,
			},
	}
}

-- Double Driller
PSO.info.potentials["earth stirrer"] = {
	
	name = "Earth Stirrer",
	desc = "Deadly Archer additionally casts Radiga in random positions around the weapon with a Technique potency of 50% that breaks Chain.",
	
	funcs = {
	
			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					local fake = p.mo.fakesaber	
					
					if p.pso.action == "dsaber_pa_deadlyarcher" and fake and fake.valid
						
						for i = 1, 5
							if timer == (TICRATE/2) + (i-1)*4
								
								for j = 1, 3
									
									local msl = PSO.lib.spawnMissile(p.mo, MT_THOK, p.drawangle, 0, 50, DMT_TECHNIQUE, 0, 0, 0)
									msl.pso.combobreaker = true	-- charge shots are always combo breakers
									msl.scale = $/3
									
									local x = fake.x + P_RandomRange(-128, 128)*FRACUNIT
									local y = fake.y + P_RandomRange(-128, 128)*FRACUNIT
									
									P_SetOrigin(msl, x, y, msl.z)
									PSO.info.photonArts["radiga"].spawnfunc(msl, 192)
								end
							end
						end
					end
							
				end,
			},
	}
}

-- Ice Lancer
PSO.info.potentials["cold snap"] = {
	
	name = "Cold Snap",
	desc = "Normal attacks also cast Barta with a Technique potency of 50% and builds chain.",
	
	funcs = {
	
			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					if p.pso.action == "spear_normal"
					
						if combo == 2 and timer == TICRATE/4
						or combo < 2 and timer == TICRATE/8

							local msl = PSO.lib.spawnMissile(p.mo, MT_BARTA, p.drawangle, (combo == 2 and 50 or 30)*FRACUNIT, 50, DMT_TECHNIQUE, 0, 0, 0)
							msl.fuse = TICRATE
							S_StartSound(p.mo, sfx_barta)
						end
					end	
				end,
			},
	}
}

-- Sneg Zapper
PSO.info.potentials["lightning discharge"] = {
	
	name = "Lightning Discharge",
	desc = "Fully charged Charge Shot has its projectile replaced with a large electrical discharge that pierces enemies with the same Ranged potency as the original projectile.",
	
	funcs = {
	
			{
				hookname = "playerActionOverride",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					if p.pso.action == "rifle_charge"
						
						-- rifle is always weapon slot 1
						local pos = p.pso.weaponpos[1]
						local potency = PSO.info.photonArts[p.pso.action].potency
						local wep = PSO.lib.getPlayerActiveWeapon(p)
						local winfo = PSO.lib.itemInfo(wep)
						--print(timer)
						-- set new weapon pos:
						local ang = p.drawangle - ANG1*30
						local x = 24*cos(ang)
						local y = 24*sin(ang)
						local mo = p.mo
						
						local chargethreshold = PSO.lib.hasSkill(p, "ra_quickload") and TICRATE*2/3 or TICRATE
						
						if timer >= chargethreshold
						and timer < TICRATE*20
						and not hold
							-- just let it go.
							S_StopSound(mo)
							p.pso.actiontimer = timer < chargethreshold and TICRATE*20 or TICRATE*30	-- first firing state.
							pos[6] = timer < chargethreshold and ANG1*40 or ANG1*90
							
							local msl = PSO.lib.spawnMissile(mo, MT_ENERGYBALL, p.drawangle, 70*FRACUNIT, potency, DMT_RANGED, pos[2], pos[3], -mo.scale*30)
							msl.pso.combobreaker = true	-- charge shots are always combo breakers
							msl.color = SKINCOLOR_YELLOW
							msl.flags = $|MF_NOCLIP|MF_NOCLIPHEIGHT
							msl.colorized = true
							msl.scale = mo.scale*3/2
							
							PSO.lib.aimMissile(msl, p.pso.target)
							
							p.pso.cancancel = TICRATE/4
							
							local range = wep.range or 512
							msl.fuse = FixedDiv(range*FRACUNIT, max(1, FixedHypot(msl.momx, msl.momy)))/FRACUNIT

							if winfo.sound
								S_StartSound(mo, winfo.sound)
							end
							S_StartSound(mo, sfx_s3k54)

							msl.pso.potency = $*140/100
								
							P_InstaThrust(mo, p.drawangle, -10*FRACUNIT)
							p.skidtime = 20
							mo.saveang = p.drawangle
							p.pso.actionspeed = 2	-- (keep some actually because otherwise dodgecancel acts very weird)
							p.drawangle = mo.angle
							mo.state = S_PLAY_WALK
							
							return true
						end
					end	
				end,
			},
	}
}

-- Command Shot
PSO.info.potentials["glorious leap"] = {
	
	name = "Glorious Leap",
	desc = "Increases the height gained from performing Glorious Rain and perform an additional attack with 250% Ranged potency on liftoff.",
	
	funcs = {
	
			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					local mo = p.mo
					if p.pso.action == "shotgun_pa_gloriousrain"
					
						if timer == 1
							local hb = PSO.lib.spawnHurtBox(mo.x, mo.y, mo.z - mo.scale*32, 320<<FRACBITS, 120<<FRACBITS, mo, 250, DMT_RANGED, 8)
							hb.pso.combobreaker = true
							S_StartSound(mo, sfx_s3k54)
								
							p.pso.cancancel = TICRATE/8
								
							for j = 1, 2
								for i = 1, 16
									local s = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
									s.angle = 360/16 *i *ANG1 + ANG1*90
									s.scale = FRACUNIT/4
									s.destscale = FRACUNIT*4
									s.fuse = 16
									s.tics = 16
									s.frame = $|FF_PAPERSPRITE
									s.color = SKINCOLOR_RED
									P_InstaThrust(s, 360/16 *i *ANG1, FRACUNIT*(j == 1 and 48 or 24))
								end	
							end
							
							mo.momz = FRACUNIT*16
						end
					end	
				end,
			},
	}
}

-- Hatted Cannon
PSO.info.potentials["frozen cannonball"] = {
	
	name = "Frozen Cannonball",
	desc = "Normal attack speed -30%. Normal attack is replaced by a lobbed projectile that inflicts freeze on most non-boss enemies for 5s with the same potency as the original projectile.",
	
	funcs = {

			{
				hookname = "attackSpeed",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, spd)
							if p.pso.action == "launcher_normal"
								spd[1] = $-30
							end
				end,
			},
	
			{
				hookname = "playerActionOverride",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)

					local pos = p.pso.weaponpos[1]
					local potency = PSO.info.photonArts[p.pso.action].potency
					local wep = PSO.lib.getPlayerActiveWeapon(p)
					local winfo = PSO.lib.itemInfo(wep)
					--print(timer)
					-- set new weapon pos:
					local ang = p.drawangle - ANG1*30
					local mo = p.mo
					
					if p.pso.action == "launcher_normal"
					
						if timer == 17

							pos[6] = ANG1*30
							
							local traveltime = 50
							local target = p.pso.target
							if not target or not target.valid
								local x = mo.x + 256*cos(p.drawangle)
								local y = mo.y + 256*sin(p.drawangle)
								target = P_SpawnMobj(x, y, mo.z, MT_THOK)
								target.fuse = 2
								target.sprite = SPR_NULL
							else -- anticipate movement.
								target = P_SpawnMobj(target.x + traveltime*target.momx, target.y + traveltime*target.momy, target.z, MT_THOK)
								target.fuse = 2
								target.sprite = SPR_NULL								
							end
							
							local spd = R_PointToDist2(mo.x, mo.y, target.x, target.y) / traveltime
							
							local msl = PSO.lib.spawnMissile(mo, MT_POPSHOT, p.drawangle, spd, potency, DMT_RANGED, 0, 0, FRACUNIT*32, RT_NONE, COND_FREEZE, 100)
							PSO.lib.lobMissile(msl, target)
							msl.radius = $*5

							if PSO.lib.hasSkill(p, "ra_jellenshot")
								msl.pso.inflictjellen = TICRATE*3
							end

							if PSO.lib.hasSkill(p, "ra_zalureshot")
								msl.pso.inflictzalure = TICRATE*3
							end

							
							for i = 1, 8
								local t = P_SpawnMobj(msl.x, msl.y, msl.z, MT_THOK)
								t.color = SKINCOLOR_CYAN
								t.angle = p.drawangle + P_RandomRange(-30, 30)*ANG1
								t.momz = P_RandomRange(-10, 10)*mo.scale
								P_InstaThrust(t, t.angle, P_RandomRange(5, 16)*mo.scale)
								t.scale = $/3
								t.tics = 16
								t.fuse = 16
							end
								
							msl.pso.potency = potency	-- normals shouldn't benefit from PA%

							msl.pso.blight = p.pso.blightround
							p.pso.blightround = nil

							if winfo.sound
								S_StartSound(mo, winfo.sound)
							end

							return true
						end
					end	
				end,
			},
	}
}

-- Ice Stabberrod
PSO.info.potentials["ice wave"] = {
	
	name = "Ice Wave",
	desc = "Casting Barta additionally creates 2 extra copies of it fanning out to the side with 10% of the original's potency.",
	
	funcs = {
	
			{
				hookname = "playerTechnique",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, msl)
					
					if p.pso.action == "rod_cast" and p.pso.taction == "barta"
						
						local ta = {p.drawangle - ANG1*15, p.drawangle + ANG1*15}
						for i = 1, 2
							
							local a = p.drawangle + ANG1*90 + ANG1*180*(i-1)
							local dist = 32
							local x = msl.x + dist*cos(a)
							local y = msl.y + dist*sin(a)
							
							local msl2 = PSO.lib.spawnMissile(p.mo, MT_BARTA, ta[i], 30*FRACUNIT, msl.pso.potency/10, DMT_TECHNIQUE, 0, 0, 0)
							msl2.fuse = msl.fuse
							msl2.combobreaker = msl.combobreaker
						end
					end	
				end,
			},
	}
}

-- Winginator
PSO.info.potentials["slippery slash"] = {
	
	name = "Slippery Slash",
	desc = "The 3rd step of Normal Attack additionally provides a large burst of forward momentum.",
	
	funcs = {
	
			{
				hookname = "playerActionOverride",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)


					local pos = p.pso.weaponpos[1]
					local potency = PSO.info.photonArts[p.pso.action].potency
					local wep = PSO.lib.getPlayerActiveWeapon(p)
					local winfo = PSO.lib.itemInfo(wep)
					local mo = p.mo
					
					if p.pso.action == "saber_normal"
					
						if combo == 2

							pos[7] = {use = false}
							
							local ang = p.drawangle - ANG1*30
							local x = 24*cos(ang)
							local y = 24*sin(ang)
							
							-- further dist for the skewer:
							x = x + (min(4, timer)*cos(p.drawangle)*16)
							y = y + (min(4, timer)*sin(p.drawangle)*16)

							pos[2] = x
							pos[3] = y
							pos[4] = mo.height/2
							pos[5] = ANG1*10
							pos[6] = 0
						
							if timer == TICRATE/8
								local hb = PSO.lib.spawnHurtBox(mo.x+x, mo.y+y, mo.z, 140<<FRACBITS, 120<<FRACBITS, mo, potency*3, DMT_STRIKE, 8, RT_BLOWDOWN)
								hb.pso.potency = potency*3	-- normals shouldn't benefit from PA%
								hb.pso.ppregen = 15
								hb.pso.reactionangle = p.drawangle
								p.mo.hitbox = hb
								p.mo.lockangle = p.drawangle
								P_InstaThrust(p.mo, p.drawangle, FRACUNIT*60)

								S_StartSound(mo, sfx_s3k81)
								if winfo.sound
									S_StartSound(mo, winfo.sound)
								end	
								
							end
							
							if timer >= TICRATE/8
							and timer < TICRATE*30 -1
								p.mo.friction = FRACUNIT - 5000
								p.drawangle = p.mo.lockangle
								if p.mo.state != S_PLAY_EDGE
									p.mo.state = S_PLAY_EDGE
								end
								
								if p.mo.hitbox and p.mo.hitbox.valid
									P_SetOrigin(p.mo.hitbox, mo.x+x, mo.y+y, p.mo.z)
									p.mo.hitbox.fuse = 8
									p.mo.hitbox.tics = 8
								end

								if leveltime%6 == 0

									for i = 1, 2
											
										local efc = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
										efc.color = SKINCOLOR_CYAN
										efc.sprite = SPR_ECRC
										efc.frame = C|TR_TRANS40|FF_PAPERSPRITE
										efc.angle = p.drawangle + ANG1*90
										efc.scale = mo.scale
										efc.destscale = (mo.scale*10) *i
										efc.scalespeed = (mo.scale/4) *i
										efc.fuse = 20
										efc.tics = 20
									end	
								end
								
								if timer > TICRATE and not hold
								or timer > TICRATE*3/2
									p.pso.actiontimer = TICRATE*30
									return
								end

								return true
							end
							
						end
					end	
				end,
			},
	}
}

-- POSS Shooter
PSO.info.potentials["weak barrel"] = {
	
	name = "Weak Barrel",
	desc = "Increases normal attack speed by 50% but decreases its damage by 80%.",
	
	funcs = {

			{
				hookname = "attackSpeed",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, spd)
							if p.pso.action == "handgun_normal"
								spd[1] = $+50
							end
				end,
			},
	
			{
				hookname = "playerDamageCalc",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				inflictorindex = 3,
				func = 	function(p, mo, inflictor, dmg)
				
					if inflictor.type == MT_PSO_RIFLESHOT
					and not inflictor.pso.combobreaker
					and inflictor.fuse
						dmg[1] = $/5
					end
				end,
			},
	}
}

-- Asterod
PSO.info.potentials["solasteron"] = {
	
	name = "Solasteron",
	desc = "Extends the duration of Foverse by 300%.",
	
	funcs = {
			{
				hookname = "playerTechnique",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, msl)
					
					if p.pso.action == "wand_cast" and p.pso.taction == "foverse"
						msl.extravalue2 = 30*4*TICRATE
					end	
				end,
			},
	}
}

PSO.info.potentials["support fire"] = {
	
	name = "Support Fire",
	desc = "While targetting an enemy, automatically fires a projectile with a Ranged potency of 50 that builds chain every 2s regardless of which weapon is in use.",
	
	funcs = {
		{
			hookname = "playerThink",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						
						local wp = p.pso.weaponpos[1][1]
					
						if leveltime%(TICRATE*2) == 0 and wp and wp.valid and p.pso.target and p.pso.target.valid

							local potency = 50
							local angle = R_PointToAngle2(wp.x, wp.y, p.pso.target.x, p.pso.target.y)
							
							wp.angle = angle
							
							local msl = PSO.lib.spawnMissile(p.mo, MT_JETTBULLET, angle, 45*FRACUNIT, potency, DMT_RANGED, 0, 0, 0)
							P_SetOrigin(msl, wp.x, wp.y, wp.z+FRACUNIT*4)
							msl.scale = $/2
							msl.target = p.mo
										
							msl.pso.potency = potency	-- normals shouldn't benefit from PA%														
							PSO.lib.aimMissile(msl, p.pso.target)	
						end
					end,
		},
	}
}

--------------------------------------------------------
-- 8* GEAR POTENTIALS --
--------------------------------------------------------

-- Seal Case
PSO.info.potentials["blunt blade"] = {
	
	name = "Blunt Blade",
	desc = "Doubles the duration of Arts Avenger's invulnerability period.",
	
	funcs = {

		{
			hookname = "playerThink",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						
						if not p.pso.action
							p.mo.expandavenger = false
						end
					end,
		},
		
		{
			hookname = "playerAction",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, timer, hold, input, combo)
					
				if p.pso.avengerguard == 10
				and not p.mo.expandavenger
					p.pso.avengerguard = 2
					p.mo.expandavenger = true
				end
			end,
		},
	}
}

-- Steel Hearts
PSO.info.potentials["revolver blade"] = {
	
	name = "Revolver Blade",
	desc = "Normal attacks shoot a projectile with a Ranged potency of 100 that builds Chain.",
	
	funcs = {
	
			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					if p.pso.action == "sword_normal"	
					
						if combo == 2 and timer >= TICRATE/8 and timer <= TICRATE/2
						or combo < 2 and timer == TICRATE/8

							local msl = PSO.lib.spawnMissile(p.mo, MT_PSO_RIFLESHOT, p.drawangle, 40*FRACUNIT, 100, DMT_RANGED, 0, 0, FRACUNIT*8)
							msl.fuse = TICRATE
							msl.color = SKINCOLOR_BLUE
							msl.scale = $/2
							S_StartSound(msl, sfx_shoot2)
							--S_StartSound(p.mo, sfx_barta)
						end
					end	
				end,
			},
	}
}

-- Monkey King Bar
PSO.info.potentials["treasure hunter"] = {
	
	name = "Treasure Hunter",
	desc = "Rare Drop Rate +0%. Picking up items or Meseta restores 25% max HP and max PP. Does not work with items dropped on purpose from players.",
	
	funcs = {
	
			{
				hookname = "playerPickupItem",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, item, global)
					
					if not global or type(item) == "number"
						PSO.lib.healPP(p.mo, p.mo, p.pso.maxpp/4)
						PSO.lib.healObject(p.mo, p.mo, p.pso.maxhp/4)
					end
				end,
			},
	}
}

-- girasole
PSO.info.potentials["weighted balance"] = {
	
	name = "Weighted Balance",
	desc = "Attack speed -30%. Attacking increases attack speed by 10% up to 6 times. Effect is lost upon switching or if no attack is performed within 8 seconds.",
	
	funcs = {

		{
			hookname = "switchedWeapons",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						p.mo.girasole_speed = 0
					end,
		},
		
		{
			hookname = "playerAction",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, timer)
						if timer == 1
							p.mo.girasole_speed = $ and min(60, $+10) or 10
							p.mo.girasole_decay = 0
							--print("spd up")
						end
					end,
		},

		{
			hookname = "playerThink",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p)
						if not p.pso.action
							p.mo.girasole_decay = $ and $+1 or 1
							if p.mo.girasole_decay > TICRATE*8
								p.mo.girasole_speed = 0
								--print("spd reset")
							end	
						end
					end,
		},
		
		{
			hookname = "attackSpeed",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, spd)
						spd[1] = $ - 30 + (p.mo.girasole_speed or 0)
					end,
			},		
		
	}
}

-- Longinus
PSO.info.potentials["photon resurrection"] = {
	
	name = "Photon Resurrection",
	desc = "When PP falls below 20% and HP is above 1, sacrifice 50% of your remaining HP to recover all of your max PP. 20s cooldown. Effect starts 5s after weapon is equipped.",
	
	funcs = {
		
		{
			hookname = "switchedWeapons",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						p.mo.longinus_wait = 0
					end,
		},		
		
		{
			hookname = "playerThink",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						
						p.mo.longinus_wait = $ and $+1 or 1
						if p.mo.longinus_cooldown
							p.mo.longinus_cooldown = $-1
						end
						
						if p.mo.longinus_wait > TICRATE*5
						and not p.mo.longinus_cooldown
							--print("OK")
							-- visual to indicate it's ready.
							local a = leveltime*ANG1*3
							if leveltime&1
								for i = 1, 2
									local x = p.mo.x + 48*cos(a)
									local y = p.mo.y + 48*sin(a)
									
									local s = P_SpawnMobj(x, y, p.mo.z + FRACUNIT*8, MT_SUPERSPARK)
									s.color = SKINCOLOR_CYAN
									s.colorized = true
									s.momx = P_RandomRange(-1, 1)*FRACUNIT
									s.momy = P_RandomRange(-1, 1)*FRACUNIT
									s.momz = P_RandomRange(-1, 1)*FRACUNIT
									
									a = $+ ANG1*180
								end
							end
							
							if p.pso.pp < p.pso.maxpp/5
							and p.pso.hp > 1
								PSO.lib.damageObject(p.mo, nil, p.pso.hp/2)
								PSO.lib.healPP(p.mo, p.mo, p.pso.maxpp)
								p.mo.longinus_cooldown = TICRATE*20
							end
						end
					end,
		},
	}
}

-- Basilleon
PSO.info.potentials["clean cut"] = {
	
	name = "Clean Cut",
	desc = "When performing charged Scythe Cleave, removes the delay for the additional attack.",
	
	funcs = {
	
			{
				hookname = "playerActionOverride",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)

					local pos = p.pso.weaponpos[1]
					local potency = PSO.info.photonArts[p.pso.action].potency
					local wep = PSO.lib.getPlayerActiveWeapon(p)
					local winfo = PSO.lib.itemInfo(wep)
					local mo = p.mo
					
					if p.pso.action == "spear_pa_scythecleave"	
						local t = timer - TICRATE*5

						if t == TICRATE/4
						and p.mo.cleavemode	-- fully charged
							local x = mo.x + FixedMul(mo.scale, 100*cos(p.drawangle))
							local y = mo.y + FixedMul(mo.scale, 100*sin(p.drawangle))
								
							--(x, y, z, w, t, source, potency, dtype, duration)
							local hb = PSO.lib.spawnHurtBox(x, y, mo.z - mo.scale*64, 152<<FRACBITS, 140<<FRACBITS, mo, potency/3, DMT_STRIKE, 8)
							hb.angle = p.drawangle
							hb.pso.combobreaker = true
								
							local trail = PSO.lib.spawnMissile(mo, MT_PSO_SPEARSCYTHE, p.drawangle, 0, potency*2/3, DMT_STRIKE, 0, 0, 0)
							P_SetOrigin(trail, hb.x, hb.y, hb.z + mo.scale*32)
							trail.extravalue1 = 1
							trail.scale = $*5/2
							trail.color = winfo.effectcolor or SKINCOLOR_CYAN
								
							if p == consoleplayer
								P_StartQuake(FRACUNIT*40, 4)
							end
							
							return true
						end
					end	
				end,
			},
	}
}

-- Full Cylinder
PSO.info.potentials["rafale"] = {
	
	name = "Rafale",
	desc = "Increases normal attack speed by 20% for every shot after the first.",
	
	funcs = {
	
			{
				hookname = "attackSpeed",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, spd)
							if p.pso.action == "rifle_normal"
							and p.pso.actioncombo
								spd[1] = $+20
							end
				end,
			},
	}
}

-- Assassin
PSO.info.potentials["decisive opener"] = {
	
	name = "Decisive Opener",
	desc = "Damage +300% if the enemy's HP is full. Only works for the first hit of any given attack.",
	
	funcs = {
	
			{
				hookname = "playerDamageCalc",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				inflictorindex = 3,
				func = 	function(p, mo, inflictor, dmg)
				
					if mo and mo.valid and mo.pso and mo.pso.hp == mo.pso.maxhp
						dmg[1] = $*400/100
					end
				end,
			},
	}
}

-- Heimdall
PSO.info.potentials["huge capacity"] = {
	
	name = "Huge Capacity",
	desc = "Triples the bonus PP earned from the Chain Reactor Type Skill.",
	
	funcs = {

			{
				hookname = "switchedWeapons",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p)
							PSO.lib.playerCalcStats(p, p.pso.level)
						end,
			},	

			{
				hookname = "switchingWeapons",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p)
							p.mo.switching = true	-- ensures we drop the buff
							PSO.lib.playerCalcStats(p, p.pso.level)
						end,
			},	
	
			{
				hookname = "playerCalcStats",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p, level, stats)
					
					
					if p.pso.chainreactor
					and not p.mo.switching
						p.pso.maxpp = $+ p.pso.chainreactor*2
					end
					p.mo.switching = nil
				end,
			},

			{
				hookname = "playerThink",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p)
							if p.pso.prevchainreactor != p.pso.chainreactor
								PSO.lib.playerCalcStats(p, p.pso.level)
							end
							p.pso.prevchainreactor = p.pso.chainreactor
						end,
			},
			
	}
}

-- Wide Smack
PSO.info.potentials["traditional slugs"] = {
	
	name = "Traditional Slugs",
	desc = "Increases the damage of Slug Burst's explosions by 33%.",
	
	funcs = {
		-- exceptionally handled directly in the object's thinker for simplicity's sake.
	}
}

-- Saint Kilda
PSO.info.potentials["perfect keeper"] = {
	
	name = "Perfect Keeper",
	desc = "Attack speed +20% while HP is full and -10% otherwise.",
	
	funcs = {	
	
			{
				hookname = "attackSpeed",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, spd)
							spd[1] = $+(p.pso.hp >= p.pso.maxhp and 20 or -10)
				end,
			},
	}
}

-- Photon Launcher
PSO.info.potentials["density disruption"] = {
	
	name = "Density Disruption",
	desc = "On Chain break, inflict 5 seconds of Shock to non-boss enemies",
	
	funcs = {	
				
				-- "playerChainBreak", p, mo, inflictor, damage, firstchain)
			{
				hookname = "playerChainBreak",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, damage, firstchain)
							if not firstchain
							and not mo.pso.boss
								PSO.lib.inflictStatus(mo, COND_SHOCK, 5*TICRATE)
							end
				end,
			},
	}
}

-- Red Processor
PSO.info.potentials["optimization protocol"] = {
	
	name = "Optimization Protocol",
	desc = "Attack speed +20% and PP cost -50% when casting the same Technique repeatedly. Attack speed -50% when first casting a different Technique.",
	
	funcs = {

			{
				hookname = "switchedWeapons",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p)
							p.mo.processorcast = 0
							p.mo.processoraction = nil
						end,
			},		
	
			{
				hookname = "attackSpeed",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, spd)
							spd[1] = $+(p.mo.processorcast or 0)
				end,
			},

			{
				hookname = "PPCost",			-- hook to register into
				type = POTTYPE_ONLYACTIVE,		-- scope, when should this run?
				playerargindex = 1,				-- tells us which argument of the function the player is. (1 by default)
				func = 	function(p, cost, action)		-- function to run akin to the hook.
							p.mo.processorcast = 0
							if p.mo.processoraction == action
								cost[1] = $/2
								p.mo.processorcast = 20
							elseif p.mo.processoraction
								p.mo.processorcast = -50
							end
							
							p.mo.processoraction = action
						end,
			},
			
	}
}

-- Clarita Visas
PSO.info.potentials["clear intention"] = {
	
	name = "Clear Intention",
	desc = "Adds a status condition cleansing effect to Resta and increases its healing potency by 50%.",
	
	funcs = {
			{
				hookname = "playerTechnique",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, msl)
					
					if p.pso.taction == "resta"
						msl.pso.potency = $*3/2

						local range = 768
						if PSO.lib.hasSkill(p, "fo_territoryburst")
							range = $*3
						end			
						
						for p2 in players.iterate do
							
							if p2 and p2.mo and p2.mo.valid
							and R_PointToDist2(p.mo.x, p.mo.y, p2.mo.x, p2.mo.y) < range*p.mo.scale
								PSO.lib.removeStatus(p2.mo)
								p2.pso.jellen = 0
								p2.pso.zalure = 0
								-- does not remove injury.
							end
						end
						
					end
				end,
			},
	}
}

-- Sarblight
PSO.info.potentials["crimson twinkle"] = {
	
	name = "Crimson Twinkle",
	desc = "Increase damage dealt by 5% when under the effects of Shifta. Applies to both weapons in the palette.",
	
	funcs = {
	
			{
				hookname = "playerDamageCalc",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg)
				
					if p.pso.shifta
						dmg[1] = $*105/100
					end
				end,
			},
	}
}

-- Forblight
PSO.info.potentials["azure twinkle"] = {
	
	name = "Azure Twinkle",
	desc = "Decreases PP consumption by 20% when under the effects of Deband. Applies to both weapons in the palette.",
	
	funcs = {
	
			{
				hookname = "PPCost",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p, cost)
				
					if p.pso.deband
						cost[1] = $*4/5
					end
				end,
			},
	}
}

-- Yasminkov... I forgot the number man fuck you
PSO.info.potentials["renowned consistency"] = {
	
	name = "Renowned Consistency",
	desc = "Increases PB gauge generation by 15% when triggering Zero Range or Sharp Shooter.",
	
	funcs = {
	
			{
				hookname = "playerPBGeneration",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg, pb)
					
					if inflictor and inflictor.valid and inflictor.pso and inflictor.pso.origin
						
						local source = inflictor.target
						local range = FixedHypot(FixedHypot(inflictor.pso.origin[1] - inflictor.x, inflictor.pso.origin[2] - inflictor.y), inflictor.pso.origin[3] - inflictor.z)
						
						if range < PSO.info.CLOSERANGE*FRACUNIT
						and PSO.lib.hasSkill(source.player, "ra_zerorange")
						
						or range > PSO.info.LONGRANGE*FRACUNIT
						and PSO.lib.hasSkill(source.player, "ra_sharpshooter")
							
							pb[1] = $*115/100
							
						end
					end
				end,
			},

			/*{
				hookname = "playerVariance",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, minvar, maxvar)
					minvar[1] = 100
					maxvar[1] = 100
				end,
			},*/
			
	}
}

PSO.info.potentials["oath to a new dawn"] = {
	
	name = "Oath to a New Dawn",
	desc = "When hitting enemy weaknesses, recover 2 PP once per attack and also generate 25% more PB gauge for every hit.",
	
	funcs = {
	
			{
				hookname = "playerPBGeneration",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg, pb)
				
					local wp = inflictor.pso.savewpn
					if wp
						if wp.element == mo.pso.weakness
						or wp.element == mo.pso.weakness2
						or inflictor.pso.element == mo.pso.weakness
						or inflictor.pso.element == mo.pso.weakness2
							pb[1] = $*5/4
						end
					end
				end,
			},

			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer)
				
					if timer == 1
						p.mo.sarcueidok = true
					end
				end,
			},

			{
				hookname = "playerDamage",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg)
				
					local wp = inflictor.pso.savewpn
					if wp
						if wp.element == mo.pso.weakness
						or wp.element == mo.pso.weakness2
						or inflictor.pso.element == mo.pso.weakness
						or inflictor.pso.element == mo.pso.weakness2
							if p.mo.sarcueidok
								PSO.lib.healPP(p.mo, p.mo, 2, true)
								p.mo.sarcueidok = nil
							end
						end
					end
				end,
			},
	}
}

-- Windmill
PSO.info.potentials["amplified support"] = {
	
	name = "Amplified Support",
	desc = "Attack speed -10%. Extends the duration of Shifta and Deband to 5:00 when using them.",
	
	funcs = {
			{
				hookname = "playerTechnique",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, msl)
					
					if p.pso.taction == "shifta"
					or p.pso.taction == "deband"

						local range = 768
						if PSO.lib.hasSkill(p, "fo_territoryburst")
							range = $*3
						end			
						
						for p2 in players.iterate do
							
							if p2 and p2.mo and p2.mo.valid
							and R_PointToDist2(p.mo.x, p.mo.y, p2.mo.x, p2.mo.y) < range*p.mo.scale
								if p.pso.taction == "shifta"
									p2.pso.shifta = 300*TICRATE
								else
									p2.pso.deband = 300*TICRATE
								end
							end
						end
					end
				end,
			},

			{
				hookname = "attackSpeed",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, spd)
							spd[1] = $-10
				end,
			},
			
	}
}

-- Hexigraph
PSO.info.potentials["targetted support"] = {
	
	name = "Targetted Support",
	desc = "Gain 3 seconds of Shifta & Deband when hitting enemy weaknesses. Applies to both weapons in the palette.",
	
	funcs = {

			{
				hookname = "playerDamage",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg)
				
					local wp = inflictor.pso.savewpn
					if (wp.element == mo.pso.weakness or inflictor.pso.element == mo.pso.weakness)
					or (wp.element == mo.pso.weakness2 or inflictor.pso.element == mo.pso.weakness2)
						--PSO.lib.healPP(p.mo, p.mo, 2, true)
						PSO.lib.addShifta(p.mo, TICRATE*3)
						PSO.lib.addDeband(p.mo, TICRATE*3)
					end
				end,
			},
	}
}

-- RP. Crawla
PSO.info.potentials["full steam ahead"] = {
	
	name = "Full Steam Ahead",
	desc = "Increases movement speed to match that of Photon Dash when casting Techniques. Applies to both weapons in the palette.",
	
	funcs = {
	
			{
				hookname = "playerAction",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)

					if p.pso.taction
						p.pso.actionspeed = 100
						p.normalspeed = FRACUNIT*26
					end
				end,
			},
	}
}

--------------------------------------------------------
-- 9* GEAR POTENTIALS --
--------------------------------------------------------

-- Last Survivor
PSO.info.potentials["to the edge"] = {
	
	name = "To The Edge",
	desc = "Negates all healing and survive with 1 HP from otherwise lethal damage once per quest. At 1 HP, Photon Arts cost no PP.",
	
	funcs = {
	
			{
				hookname = "PPCost",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, cost)
				
					if p.pso.hp == 1
						cost[1] = 0
					end
				end,
			},	

			{
				hookname = "playerDamaged",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				inflictorindex = 2,
				func = 	function(p, inflictor, dmg)
					
					if not p.mo.totheedge	
					and not p.pso.hp	-- that killed me?!
						p.pso.hp = 1
						p.mo.totheedge = true

						local s = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_THOK)
						s.sprite = SPR_FORC
						s.frame = A|FF_ADD
						s.destscale = FRACUNIT*8
						s.scalespeed = FRACUNIT/7
						s.fuse = 16
						s.tics = s.fuse
						
						P_FlashPal(p, 1, 3)
						
					end
				end,
			},

		{
				hookname = "playerHeal",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, inflictor, heal)
							
							if p.pso.hp	-- we aren't dead
								heal[1] = 0	-- we can't heal for shit!!
							end
					end,
			},
			
	}
}

-- Zanba
PSO.info.potentials["critical ignorance"] = {
	
	name = "Critical Ignorance",
	desc = "On Critical Hit, cleanse negative effects such as Status Effects, Jellen, Zalure, Mate Sickness and Reverser Weakness off of yourself.",
	
	funcs = {

			{
				hookname = "playerDamage",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg, skipchain, crit)
				
					if crit
						PSO.lib.removeStatus(p.mo)
						p.pso.jellen = 0
						p.pso.zalure = 0
						p.pso.matesickness = 0
						p.pso.weakness = 0
						p.pso.brink = 0
					end
				end,
			},
	}
}

-- Eternal Heat
PSO.info.potentials["burning resolve"] = {
	
	name = "Burning Resolve",
	desc = "On Chain Break, inflict self with Burn for 5 seconds. While burning, Photon Arts cost no PP.",
	
	funcs = {	
				
				-- "playerChainBreak", p, mo, inflictor, damage, firstchain)
			{
				hookname = "playerChainBreak",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, damage, firstchain)
						if not firstchain
							PSO.lib.inflictStatus(p.mo, COND_BURN, 5*TICRATE)
						end
				end,
			},

			{
				hookname = "PPCost",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, cost)
				
					if p.pso.condition == COND_BURN
						cost[1] = 0
					end
				end,
			},
	}
}

-- D. Saber Legacy
PSO.info.potentials["relentless combo"] = {
	
	name = "Relentless Combo",
	desc = "Damage -10%. Using the same attack multiple times in a row increases its damage by 10% up to 4 times.",
	
	funcs = {

		{
			hookname = "switchedWeapons",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						p.mo.dsaber_damage = -10
						p.mo.dsaber_lastact = nil
					end,
		},
		
		{
			hookname = "playerAction",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, timer)
						if timer == 1
							
							if p.mo.dsaber_lastact ~= p.pso.action
								p.mo.dsaber_damage = -10
								S_StartSound(p.mo, sfx_kc65, p)
							else
								p.mo.dsaber_damage = min(30, $+10)
							end
							
							p.mo.dsaber_lastact = p.pso.action
						end
					end,
		},	

		{
				hookname = "playerDamageCalc",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				inflictorindex = 3,
				func = 	function(p, mo, inflictor, dmg)
				
					if p.mo.dsaber_damage == nil return end
					dmg[1] = $*(p.mo.dsaber_damage + 100)/100
				end,
		},
	}
}

-- Brionac
PSO.info.potentials["photon leech"] = {
	
	name = "Photon Leech",
	desc = "During Chain Break, normal attack PP recovery +300%.",
	
	funcs = {	
			{
				hookname = "playerAPPRegen",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, arg)
							if mo.pso and mo.pso.chainbreak
								arg[1] = $*4
							end
				end,
			},
	}
}

-- Meteor Lancer
PSO.info.potentials["comet piercer"] = {
	
	name = "Comet Piercer",
	desc = "Increases attack speed by 30% when using Million Skewer.",
	
	funcs = {

			{
				hookname = "attackSpeed",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, spd)
							if p.pso.action == "spear_pa_millionskewer"
								spd[1] = $+30
							end
				end,
			},
	}
}

-- Strauss
PSO.info.potentials["pure shell"] = {
	
	name = "Pure Shell",
	desc = "Attacks siphon targets' Shifta and Deband. Generate 10% Photon Blast when succesfully siphonning an effect.",
	
	funcs = {

			{
				hookname = "playerDamage",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg)
					
					local shifta = mo.pso.shifta
					local deband = mo.pso.deband
					
					mo.pso.shifta = 0
					mo.pso.deband = 0
					
					if shifta
						p.pso.jellen = 0
						PSO.info.addShifta(p.mo, shifta)
						if PSO.lib.playerHasPB(p)
							p.pso.pb = min(PSO.info.MAXPB, $ + PSO.info.MAXPB/10)
						end
					end
					if deband
						p.pso.zalure = 0
						PSO.info.addDeband(p.mo, deband)
						if PSO.lib.playerHasPB(p)
							p.pso.pb = min(PSO.info.MAXPB, $ + PSO.info.MAXPB/10)
						end
					end
				end,
			},
	}
}

-- Crush Bullet
PSO.info.potentials["overwhelming presence"] = {
	
	name = "Overwhelming Presence",
	desc = "Damage resistance +30% and knockback immunity against non-boss enemies that are below 80% HP.",
	
	funcs = {

			{
				hookname = "playerReaction",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				inflictorindex = 3,
				func = 	function(p, mo, inflictor, arg)
				
					if inflictor.target and inflictor.target.pso and inflictor.target.pso.hp < inflictor.target.pso.maxhp*8/10
						arg[1] = RT_NONE
					end
				end,
			},
	
			{
				hookname = "playerDamagedCalc",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				inflictorindex = 3,
				func = 	function(p, mo, inflictor, dmg)
				
					if inflictor.target and inflictor.target.pso and inflictor.target.pso.hp < inflictor.target.pso.maxhp*8/10
						dmg[1] = $*7/10
					end
				end,
			},
	}
}

-- Barett
PSO.info.potentials["well worn overheating"] = {
	
	name = "Well Worn Overheating",
	desc = "Attack Speed -20%. Attacking increases Attack Speed by 10% and PP consumption by 5% up to 5 times. Effect is lost when swapping weapons.",
	
	funcs = {

		{
			hookname = "switchedWeapons",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						p.mo.overheat = 0
					end,
		},
		
		{
			hookname = "playerAction",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, timer)
						if timer == 1
							p.mo.overheat = min(5, $+1)
						end
					end,
		},
		
		{
			hookname = "attackSpeed",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, spd)
						spd[1] = $ - 20 + (p.mo.overheat*10)
					end,
			},		

		{
			hookname = "PPCost",			-- hook to register into
			type = POTTYPE_ONLYACTIVE,		-- scope, when should this run?
			playerargindex = 1,				-- tells us which argument of the function the player is. (1 by default)
			func = 	function(p, cost)		-- function to run akin to the hook.
						cost[1] = $ * (100 + 5*p.mo.overheat)/100
					end,
		},
		
	}
}

-- Charge Howzer
PSO.info.potentials["charged intent"] = {
	
	name = "Charged Intent",
	desc = "When using a Photon Art, frontal attacks will be negated for 1 second.",
	
	funcs = {
		
		{
			hookname = "playerAction",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, timer)
						if timer == 1
							p.mo.howzermit = true
							p.pso.mitigations = {
								{p.mo.angle - ANG1*60, p.mo.angle + ANG1*60}
							}
						elseif timer > TICRATE and p.mo.howzermit
							p.mo.howzermit = nil
							p.pso.mitigations = {}
						end
					end,
		},
	}
}

-- Battle Verge
PSO.info.potentials["close quarters"] = {
	
	name = "Close Quarters",
	desc = "Casting Techniques will also unleash a short ranged attack with a Melee potency of 100% that builds chain and restores 8 PP.",
	
	funcs = {
		
		{
			hookname = "playerAction",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, timer)

						local mo = p.mo
						local pos = p.pso.weaponpos[1]
						local tech = PSO.info.photonArts[p.pso.taction]
						local wep = PSO.lib.getPlayerActiveWeapon(p)
						
						if timer == 1
							mo.prevrodpos = nil
						end
						
						if not tech
							return true
						end
						local casttime = tech.casttime or TICRATE/2
						local swingtime = TICRATE*10
						
						-- force PB reduces cast time
						if PSO.lib.usingPB(p) == "fo_photonflare"
							casttime = max(1, $*8/10)
						end
						
						-- phantom cast nullifies it!
						if p.pso.phantomcast
						and timer == 1
							casttime = 1
						end
						
						-- fo TAJA does that too!
						if PSO.lib.hasSkill(p, "fo_techartsalt")
						and p.pso.lastelement and p.pso.lastelement ~= tech.element
							casttime = max(1, $/2)
						end
						
						if timer <= casttime
							
							local x = FixedMul(mo.scale, 8*cos(p.drawangle - ANG1*40))
							local y = FixedMul(mo.scale, 8*sin(p.drawangle - ANG1*40))
							local z = FixedDiv(mo.height*12/10, mo.scale)
							
							pos[2] = x
							pos[3] = y
							pos[4] = z
							pos[5] = ANG1*90
							pos[6] = ANG1*90
							
							p.pso.cancancel = 1
							
						end
						
						-- swing the rod and cast the tech!
						if timer >= swingtime
							if mo.prevrodpos
								PSO.lib.sword_trail(mo, pos, mo.prevrodpos)
							end
							
							if timer == swingtime+TICRATE/4
								local x = mo.x + FixedMul(mo.scale, 100*cos(p.drawangle))
								local y = mo.y + FixedMul(mo.scale, 100*sin(p.drawangle))
								
								--(x, y, z, w, t, source, potency, dtype, duration)
								local hb = PSO.lib.spawnHurtBox(x, y, mo.z-mo.scale*64, 140<<FRACBITS, 128<<FRACBITS, mo, 100, DMT_STRIKE, 8)
								hb.pso.ppregen = 8
								hb.pso.locks = 4
								hb.angle = p.drawangle
								hb.pso.hitlag = 4
								hb.pso.screenshake = {intensity = FRACUNIT*8, time=5}
								
								S_StartSound(mo, sfx_sword)
							end
							
							mo.prevrodpos = {
								pos[2],
								pos[3],
								pos[4],
								pos[6],	-- rollangle							
							}
						end
					end,
		},
	}
}

-- Brave Hammer
PSO.info.potentials["photon saviour"] = {
	
	name = "Photon Saviour",
	desc = "Gain frontal guard while charging Techniques. On block, Technique potency +15%. Lose 20% of damage taken as PP.",
	
	funcs = {
	
		{
			hookname = "playerAction",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, timer)
						local pos = p.pso.weaponpos[1]
						
						if p.mo.rodmit
							if leveltime & 1
								p.mo.colorized = true
								p.mo.color = SKINCOLOR_RED
							else
								p.mo.colorized = false
								p.mo.color = p.skincolor
							end
						end
						
						if timer == 1
							p.mo.rodmit = true
							p.pso.mitigations = {
								{p.mo.angle - ANG1*60, p.mo.angle + ANG1*60}
							}
							
							local s = P_SpawnMobjFromMobj(p.mo, 0, 0, 0, MT_THOK)
							s.sprite = SPR_FORC
							s.frame = (leveltime%20 /2) | FF_TRANS50
							s.colorized = true
							s.color = SKINCOLOR_CRIMSON
							s.tics = -1
							s.fuse = 7
							s.scalespeed = FRACUNIT/2
							s.destscale = FRACUNIT*8	
							
							--print("mit on")
						elseif p.mo.rodmit and (pos[7] and pos[7].use)
							p.mo.rodmit = nil
							p.pso.mitigations = {}
							p.mo.colorized = false
							p.mo.color = p.skincolor
							--print("mit off")
						end
					end,
		},

		{
			hookname = "playerBlock",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			inflictorindex = 2,
			func = 	function(p, inflictor, source)
				
				if p.mo.rodmit
					local t = PSO.info.photonArts[p.pso.taction]
					if not t or not t.cost return end	--???
					--PSO.lib.healPP(p.mo, p.mo, t.cost*3)
					p.mo.braveboost = true
					
					S_StartSoundAtVolume(p.mo, sfx_cburst, 127)
								
					for i = 1, 8
						local s = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_SUPERSPARK)
						s.destscale = 1
						s.momx = P_RandomRange(-32, 32)*FRACUNIT
						s.momy = P_RandomRange(-32, 32)*FRACUNIT
						s.momz = P_RandomRange(-32, 32)*FRACUNIT
						s.colorized = true
						s.color = SKINCOLOR_YELLOW
					end
				end
			end,
		},

		{
			hookname = "playerTechnique",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, msl)
					
				if p.mo.braveboost
					p.mo.braveboost = nil
					msl.pso.potency = $*115/100
					--print("boosted")
				end	
			end,
		},

		{
			hookname = "playerDamaged",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			inflictorindex = 2,
			func = 	function(p, inflictor, dmg)
					
				PSO.lib.damagePP(p.mo, inflictor, max(1, dmg/5))				
			end,
		},
	}
}

-- Red Saber
PSO.info.potentials["s-red"] = {
	
	name = "S-Red",
	desc = "Performing the final combo of the Normal Attack grants 20 seconds of Shifta and Deband, overriding the effects of Jellen and Zalure.",
	
	funcs = {
		
			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					if timer == 1 and combo == 2
						p.pso.jellen = 0
						p.pso.zalure = 0
						PSO.lib.addShifta(p.mo, TICRATE*20)
						PSO.lib.addDeband(p.mo, TICRATE*20)
					end
				end,
			},
	}
}

-- Grand Cross
PSO.info.potentials["pure blade"] = {
	
	name = "Pure Blade",
	desc = "Attacks siphon targets' Shifta and Deband. Generate 10% Photon Blast when succesfully siphonning an effect.",
	funcs = PSO.info.potentials["pure shell"].funcs
}

-- Varista
PSO.info.potentials["high voltage shell"] = {
	
	name = "High Voltage Shell",
	desc = "On Chain break, inflict 5 seconds of Shock to non-boss enemies.",
	
	funcs = {	
				
				-- "playerChainBreak", p, mo, inflictor, damage, firstchain)
			{
				hookname = "playerChainBreak",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, damage, firstchain)
							if not firstchain and not mo.pso.boss
								PSO.lib.inflictStatus(mo, COND_SHOCK, 5*TICRATE)
							end
				end,
			},
	}
}

-- Bravace
PSO.info.potentials["remote guidance"] = {
	
	name = "Remote Guidance",
	desc = "Triples the Fire Rate of the Auto Turret when deployed. Active for both weapons in the palette.",
	
	funcs = {}	-- handled in the autoturret directly.
}

-- Serdote
PSO.info.potentials["reflex jalure"] = {
	
	name = "Reflex Jalure",
	desc = "When taking at least 10 damage, cast Jellen and Zalure in response centered on your target or yourself if there is none. 0.33s cooldown.",
	
	funcs = {
	
			{
				hookname = "playerDamaged",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				--inflictorindex = 2,
				func = 	function(p, inflictor, dmg)
					
					if dmg and dmg >= 10 and (not p.mo.reflexgifoie or leveltime-p.mo.reflexjalure > (TICRATE/3))
						
						p.mo.reflexjalure = leveltime
						local origin = p.mo
						local source = p.pso.target
						if source and source.valid
							origin = source
						end

						local msl = PSO.lib.spawnMissile(p.mo, MT_THOK, p.drawangle, 0, 0, DMT_TECHNIQUE, 0, 0, 0)
										
						P_SetOrigin(msl, origin.x, origin.y, origin.z)
						PSO.info.photonArts["jellen"].spawnfunc(msl)

						local msl = PSO.lib.spawnMissile(p.mo, MT_THOK, p.drawangle, 0, 0, DMT_TECHNIQUE, 0, 0, 0)
										
						P_SetOrigin(msl, origin.x, origin.y, origin.z)					
						PSO.info.photonArts["zalure"].spawnfunc(msl)
					end
				end,
			},
	}
}

-- Rose Quartz
PSO.info.potentials["reflex gifoie"] = {
	
	name = "Reflex Gifoie",
	desc = "When taking at least 10 damage, cast Gifoie in response centered on the target or yourself if there is none. 0.33s cooldown.",
	
	funcs = {
	
			{
				hookname = "playerDamaged",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				--inflictorindex = 2,
				func = 	function(p, inflictor, dmg)

					if dmg and dmg >= 10 and (not p.mo.reflexgifoie or leveltime-p.mo.reflexgifoie > (TICRATE/3))
						
						p.mo.reflexgifoie = leveltime
						
						local origin = p.mo
						local source = p.pso.target
						if source and source.valid
							origin = source
						end
						
						local potency = PSO.info.photonArts["gifoie"].potency
						local msl = PSO.lib.spawnMissile(p.mo, MT_THOK, p.drawangle, potency, potency, DMT_TECHNIQUE, 0, 0, 0)
						msl.pso.combobreaker = true
										
						P_SetOrigin(msl, origin.x, origin.y, origin.z)
						PSO.info.photonArts["gifoie"].spawnfunc(msl)
					end
				end,
			},
	}
}

-- Omochao
PSO.info.potentials["malevolent entity"] = {
	
	name = "Malevolent Entity",
	desc = "Casting Shifta and Deband will also cast Jellen and Zalure or vice-versa.",
	
	funcs = {
	
			{
				hookname = "playerTechnique",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, msl)
					
					local lookup = {
						["shifta"] = "jellen",
						["deband"] = "zalure",
						["jellen"] = "shifta",
						["zalure"] = "deband",
					}
					
					local mo = p.mo
					
					if lookup[p.pso.taction]
						
						local t = PSO.lib.spawnMissile(p.mo, MT_THOK, 0, 0, 1, DMT_TECHNIQUE, 0, 0, 0)
						P_SetOrigin(t, mo.x, mo.y, mo.z)
						
						PSO.info.photonArts[lookup[p.pso.taction]].spawnfunc(t)
					end
				end,
			},
	}
}

--------------------------------------------------------
-- 10* GEAR POTENTIALS --
--------------------------------------------------------

-- Monado (lol)
PSO.info.potentials["blade buster"] = {
	
	name = "Blade Buster",
	desc = "Skips the first two swings of Over End.",
	
	funcs = {
			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					if p.pso.action == "sword_pa_overend"	
						if timer == TICRATE/5
							p.pso.actiontimer = TICRATE*3/2
						end
					end	
				end,
			},	
	},
}

-- Chainsawd
PSO.info.potentials["photon shredder"] = {
	
	name = "Photon Shredder",
	desc = "Normal attacks have additional hits but have their total potency reduced by 60%. Recover 8 HP on Critical Hit.",
	
	funcs = {
	
			{
				hookname = "playerActionOverride",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					if p.pso.action == "sword_normal"	
						local potency = PSO.info.photonArts[p.pso.action].potency
						local mo = p.mo
					
						--if combo == 2 and timer >= TICRATE/8 and timer <= TICRATE/2
						if combo < 2
						and timer == TICRATE/8
							local x = mo.x + FixedMul(mo.scale, 100*cos(p.drawangle))
							local y = mo.y + FixedMul(mo.scale, 100*sin(p.drawangle))
							
							--(x, y, z, w, t, source, potency, dtype, duration)
							local hb = PSO.lib.spawnHurtBox(x, y, mo.z-mo.scale*64, 140<<FRACBITS, 128<<FRACBITS, mo, potency*3/10, DMT_STRIKE, 8)
							hb.pso.potency = potency*3/10	-- normals shouldn't benefit from PA%
							hb.pso.ppregen = 10
							hb.pso.locks = 4
							hb.angle = p.drawangle
							hb.pso.hitlag = 4
							hb.pso.screenshake = {intensity = FRACUNIT*8, time=5}
							S_StartSound(mo, sfx_s3ka8)
							
							--p.pso.cancancel = 3
							return true
						elseif combo == 2
						and timer == TICRATE/4
							--(x, y, z, w, t, source, potency, dtype, duration)
							local hb = PSO.lib.spawnHurtBox(mo.x, mo.y, mo.z-mo.scale*64, 340<<FRACBITS, 128<<FRACBITS, mo, potency*2/10, DMT_STRIKE, 8)
							hb.pso.potency = potency*2/10	-- normals shouldn't benefit from PA%
							hb.pso.ppregen = 15
							hb.pso.locks = 6
							hb.pso.hitlag = 2
							hb.pso.screenshake = {intensity = FRACUNIT*16, time=5}						
							
							S_StartSound(mo, sfx_s3ka8)
							
							return true
						end
					end	
				end,
			},

			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					if p.pso.action == "sword_normal"	
						local potency = PSO.info.photonArts[p.pso.action].potency
						local mo = p.mo
					
						--if combo == 2 and timer >= TICRATE/8 and timer <= TICRATE/2
						if combo < 2
						and timer == TICRATE/8 + 3

						or combo == 2
						and timer >= TICRATE/4
						and timer <= TICRATE/4 + 7
						--and timer%2
						
							local x = mo.x + FixedMul(mo.scale, 100*cos(p.drawangle))
							local y = mo.y + FixedMul(mo.scale, 100*sin(p.drawangle))
							
							local pot = (combo < 2 and (potency*3/10) or (potency*2/10))
							
							--(x, y, z, w, t, source, potency, dtype, duration)
							local hb = PSO.lib.spawnHurtBox(x, y, mo.z-mo.scale*64, 140<<FRACBITS, 128<<FRACBITS, mo, pot, DMT_STRIKE, 8)
							hb.pso.potency = pot	-- normals shouldn't benefit from PA%
							hb.pso.locks = 4
							hb.angle = p.drawangle
							hb.pso.hitlag = 2
							hb.pso.screenshake = {intensity = FRACUNIT*8, time=5}
							S_StartSound(mo, sfx_s3ka8)
							
							p.pso.cancancel = 5
						end
					end	
				end,
			},

			{
				hookname = "playerDamage",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg, skipchain, crit)
				
					if crit
						PSO.lib.healObject(p.mo, p.mo, 8, true)
					end
				end,
			},			
			
	}
}

-- Dragon Slayer
PSO.info.potentials["dragon's roar"] = {
	
	name = "Dragon's Roar",
	desc = "Normal attack damage -10%. Normal attacks benefit from Chain Boost without breaking Chains, however, Photon Arts cannot be used.",
	
	funcs = {

		{
			hookname = "PPCost",			-- hook to register into
			type = POTTYPE_ONLYACTIVE,		-- scope, when should this run?
			playerargindex = 1,				-- tells us which argument of the function the player is. (1 by default)
			func = 	function(p, cost)		-- function to run akin to the hook.
						if cost[1] > 0 
							cost[1] = 99999
						end
					end,
		},

		{
				hookname = "playerDamageCalc",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				inflictorindex = 3,
				func = 	function(p, mo, inflictor, dmg)
					
					dmg[1] = $*90/100
					
					local playercount = 0
					local minfo = mo.pso
					if not minfo return end
					--print(minfo.maxhp)
					--print(minfo.chain)
					
					for player in players.iterate
						playercount = $+1
						if playercount >= #PSO.info.chain_start
							break
						end
					end
					
					local start = PSO.info.chain_start[playercount] --or PSO.info.chain_start[#PSO.info.chain_start]
					-- check increase...
					local inc = start
					
					if minfo.chain --and minfo.chain > 2
						for i = 1, minfo.chain
							local val = PSO.info.chain_increase[playercount][i]
							inc = $ + (val or PSO.info.chain_increase[playercount][#PSO.info.chain_increase[playercount]])
						end

						S_StartSoundAtVolume(mo, sfx_cburst, 127)
						
						-- get average coord
						local avgx = (mo.x/FRACUNIT + inflictor.x/FRACUNIT)/2 *FRACUNIT
						local avgy = (mo.y/FRACUNIT + inflictor.y/FRACUNIT)/2 *FRACUNIT
						local avgz = (mo.z/FRACUNIT + inflictor.z/FRACUNIT)/2 *FRACUNIT
						
						for i = 1, 8
							local s = P_SpawnMobj(avgx, avgy, avgz, MT_SONIC3KBOSSEXPLODE)
							s.destscale = 1
							s.momx = P_RandomRange(-32, 32)*FRACUNIT
							s.momy = P_RandomRange(-32, 32)*FRACUNIT
							s.momz = P_RandomRange(-32, 32)*FRACUNIT
						end
						
						--print("chain mult: "..inc)
						dmg[1] = $ * inc / 100
					end
				end,
		},		
	
	},
}

-- Vivienne
PSO.info.potentials["still lily"] = {
	
	name = "Still Lily",
	desc = "PP Recovery +15% and PP Cost -15% when Photon Blast gauge is full. Effects are reversed if the gauge is less than half full.",
	
	funcs = {
		-- p.pso.pb >= PSO.info.MAXPB
		{
			hookname = "PPCost",			-- hook to register into
			type = POTTYPE_ONLYACTIVE,		-- scope, when should this run?
			playerargindex = 1,				-- tells us which argument of the function the player is. (1 by default)
			func = 	function(p, cost)		-- function to run akin to the hook.
						if cost[1] > 0 
							if p.pso.pb >= PSO.info.MAXPB
								cost[1] = max(1, $*85/100)
							elseif p.pso.pb < PSO.info.MAXPB/2
								cost[1] = $*115/100
							end
						end
					end,
		},

		{
			hookname = "playerAPPRegen",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, mo, inflictor, arg)
						if p.pso.pb >= PSO.info.MAXPB
							arg[1] = $*115/100
						elseif p.pso.pb < PSO.info.MAXPB/2
							arg[1] = $*85/100
						end
				end,
		},
		
		{
			hookname = "playerThink",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						
						if (p.pso.pb >= PSO.info.MAXPB or p.pso.pb < PSO.info.MAXPB/2)
						and (not p.mo.lily or not p.mo.lily.valid)
							
							local s = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_THOK)
							s.sprite = SPR_SIGI
							s.frame = B|FF_ADD
							s.scale = 1
							s.tics = -1
							s.destscale = FRACUNIT/2
							s.scalespeed = FRACUNIT/24
							s.renderflags = $|RF_FLOORSPRITE|RF_NOSPLATBILLBOARD
							s.color = p.pso.pb >= PSO.info.MAXPB and SKINCOLOR_SALMON or SKINCOLOR_GREY
							--s.fuse = PSO.info.PBDURATION +10
							p.mo.lily = s
						elseif p.mo.lily and p.mo.lily.valid
							P_MoveOrigin(p.mo.lily, p.mo.x, p.mo.y, p.mo.z)
							
							if (p.pso.pb >= PSO.info.MAXPB or p.pso.pb < PSO.info.MAXPB/2)
								p.mo.lily.fuse = 11
							end
						end
			
					end,
		},
		
	}
}

-- Double Cannon
freeslot("MT_PSO_DETONATIONSHOT")
mobjinfo[MT_PSO_DETONATIONSHOT] = {
	spawnstate = S_INVISIBLE,
	spawnhealth = 1000,
	radius = 16<<FRACBITS,
	height = 16<<FRACBITS,
	flags = MF_NOGRAVITY|MF_MISSILE,
}

addHook("MobjThinker", function(mo)
	
	mo.sprite = SPR_SSPK
	mo.frame = (leveltime%12 /4)|FF_FULLBRIGHT
	mo.color = SKINCOLOR_PURPLE
	mo.colorized = true
	mo.tics = -1
	
	-- krilling myself
	if not mo.target or not mo.target.valid
		P_RemoveMobj(mo)
		return
	end
	
	if leveltime % (TICRATE/8) == 0
		local gale = P_SpawnMobj(mo.x, mo.y, mo.z, MT_HUNTERGALE)
		gale.target = mo
		gale.fuse = TICRATE
		gale.flags2 = $|MF2_DONTDRAW
		gale.color = SKINCOLOR_PURPLE
	end
	
	
	if leveltime%(TICRATE/4) == 0
		local hb = PSO.lib.spawnHurtBox(mo.x, mo.y, mo.z-mo.height, 256<<FRACBITS, 256<<FRACBITS, mo.target, mo.pso.potency, DMT_TECHNIQUE, TICRATE/2, RT_NONE)
		hb.pso.combobreaker = mo.pso.combobreaker
		hb.pso.element = mo.pso.element
		hb.pso.potency = mo.pso.potency
		hb.pso.ppregen = 2
	end
end, MT_PSO_DETONATIONSHOT)

addHook("MobjCollide", function(mo1, mo2) return false end, MT_PSO_DETONATIONSHOT)
addHook("MobjMoveCollide", function(mo1, mo2) return false end, MT_PSO_DETONATIONSHOT)

PSO.info.potentials["detonation shot"] = {
	
	name = "Detonation Shot",
	desc = "Normal Attack 3 launches a slow multi-hit projectile attack with a Ranged potency of 50.",
	
	funcs = {

		{
			hookname = "playerAction",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, timer, hold, input, combo)
					
						if p.pso.action == "dsaber_normal"
							
							if combo == 2 and timer == 21

								local da = p.drawangle - ANG1*45
								for i = 1, 16
									local d = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_SPINDUST)
									P_InstaThrust(d, da, FRACUNIT*P_RandomRange(96, 128))
									d.momz = P_RandomRange(0, 4)*p.mo.scale
									d.scale = p.mo.scale*2
									d.renderflags = $|RF_PAPERSPRITE
									d.angle = da + ANG1*90
									d.tics = $ + P_RandomRange(1, 6)
									d.destscale = p.mo.scale*3
									
									da = $ + (90/16)*ANG1
								end

								local msl = PSO.lib.spawnMissile(p.mo, MT_PSO_DETONATIONSHOT, p.drawangle, 8*FRACUNIT, 50, DMT_RANGED, 0, 0, 0)
								msl.fuse = TICRATE*2
								S_StartSound(p.mo, sfx_s3k66)
								p.mo.state = S_PLAY_WALK
								
								P_InstaThrust(p.mo, p.mo.angle, -FRACUNIT*16)
								if p == consoleplayer
									P_StartQuake(FRACUNIT*40, 5)
								end
								
							end
						end	
					end,
		},
	}
}

freeslot("MT_PSO_KINSECT")
mobjinfo[MT_PSO_KINSECT] = {
	doomednum = -1,
	spawnstate = S_INVISIBLE,
	spawnhealth = 1000,
	flags = MF_NOGRAVITY,
	height = 32<<FRACBITS,
	height = 32<<FRACBITS,
}

-- returns true if we bumped into an enemy and it can be damaged!
local function kinsect_checkEnemies(mo)
	
	local retval
	
	searchBlockmap("objects", function(mo, found)
		
		--print(found.type.."|"..MT_BLUECRAWLA)
		
		if found and found.valid
		and found.type ~= MT_PLAYER	-- please don't do that you fucking tart
		and found.flags & MF_SHOOTABLE
		and not (found.flags & MF_NOBLOCKMAP)
		and not (found.flags & (MF_NOCLIP|MF_NOCLIPTHING))
		and mo.z+mo.height >= found.z
		and mo.z <= found.z+found.height
			retval = found
			return true
		end
	end, mo)
	
	return retval
end

local function kinsect_bump(mo)
	mo.bumping = true
	mo.burst = nil
	mo.momx = -mo.momx
	mo.momy = -mo.momy
	mo.momz = FRACUNIT*8
end

-- only run this when we've confirmed we CAN deal damage.
local function kinsect_damageEnemy(mo, target)
	
	local hb = PSO.lib.spawnHurtBox(target.x, target.y, target.z, 32<<FRACBITS, 32<<FRACBITS, mo.target, mo.pso.potency, DMT_MELEE, 2)
	
	if target.flags & MF_ENEMY	-- no extract from boxes
		mo.extract = $+1
	end
end

addHook("MobjThinker", function(mo)
	
	mo.sprite = SPR_BUZZ
	mo.frame = ((leveltime%10 < 5) and A or B)|FF_ADD
	mo.colorized = true
	mo.color = (mo.extract >= 4 and SKINCOLOR_RED or SKINCOLOR_TEAL)
	
	mo.tics = -1
	
	-- behaviours...
	if not mo.target or not mo.target.valid
		P_RemoveMobj(mo)
		return
	end
	
	-- bumping, just let it decelerate.
	if mo.bumping
		
		mo.momx = $*8/10
		mo.momy = $*8/10
		mo.momz = $*8/10
		mo.rollangle = $ + 	ANG1*45
		
		if FixedHypot(mo.momx, mo.momy) < FRACUNIT*2
			mo.momx = 0
			mo.momy = 0
			mo.momz = 0
			mo.bumping = nil
		end
		
		return
	end
	
	-- go back to owner
	if mo.recall
		mo.rollangle = 0
		
		local recall_duration = (TICRATE/2)
		local prog = mo.recall*FRACUNIT/recall_duration
		
		mo.momx = ease.inoutsine(prog, mo.x, mo.target.x) - mo.x
		mo.momy = ease.inoutsine(prog, mo.y, mo.target.y) - mo.y
		mo.momz = ease.inoutsine(prog, mo.z, mo.target.z+mo.target.height/2) - mo.z		
		
		mo.angle = R_PointToDist2(0, 0, mo.momx, mo.momy)
		mo.flags = $|MF_NOCLIP|MF_NOCLIPHEIGHT
		
		mo.recall = $+1
		
		-- we've done our purpose
		if mo.recall >= recall_duration
			if mo.extract >= 4
				mo.target.kinsectextract = TICRATE*30	-- 30 seconds
			end
			P_RemoveMobj(mo)
		end
		
		return
	end
	
	if mo.burst
		mo.rollangle = $ + ANG1*20
		local g = P_SpawnGhostMobj(mo)
		g.colorized = true
		
		mo.burst = $-1
		mo.attackdelay = 0
		
		if mo.burst == 1	-- stop right there
			P_InstaThrust(mo, 0, 0)
		end
		
		local scan = kinsect_checkEnemies(mo)
		if scan
			mo.tracer = scan
			kinsect_bump(mo)
			
			-- hurt the enemy!
			kinsect_damageEnemy(mo, scan)
		end
		
		return
	else
		mo.rollangle = 0
	end
	
	-- we have a valid target to attack
	if mo.tracer and mo.tracer.valid
		
		mo.attackdelay = $ and $+1 or 1
		
		-- go for the enemy!
		if mo.attackdelay > TICRATE*2
			mo.burst = TICRATE
			mo.angle = R_PointToAngle2(mo.x, mo.y, mo.tracer.x, mo.tracer.y)
			P_InstaThrust(mo, mo.angle, FRACUNIT*60)
			PSO.lib.aimMissile(mo, mo.tracer)
		end
	end
	
end, MT_PSO_KINSECT)

-- kinsect wall collision
addHook("MobjMoveBlocked", function(mo)
	if mo and mo.valid and mo.burst
		kinsect_bump(mo)
		return true
	end
end, MT_PSO_KINSECT)

local KINSECT_MAX_STAMINA = TICRATE*20

-- Hidden Cane
PSO.info.potentials["kinsect mastery"] = {
	
	name = "Kinsect Mastery",
	desc = "Attack speed -20%. Hold Normal Attack 1 to send or recall a kinsect that will attack periodically, building an extract and chain. Once the extract is built, recalling the kinsect grants Attack Speed +45% for 45s.",
	
	funcs = {
				
			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					if p.pso.action == "dsaber_normal"
						--print(timer)
						
						if not combo
						and timer == TICRATE*30 - 8
						and hold
							--print("A")
							-- go my kinsect
							if not p.mo.kinsect or not p.mo.kinsect.valid
								
								local msl = PSO.lib.spawnMissile(p.mo, MT_PSO_KINSECT, p.drawangle, 0, 1, DMT_MELEE, 0, 0, FRACUNIT*60)
								msl.extract = 0
								msl.burst = TICRATE*3/2
								P_InstaThrust(msl, p.drawangle, FRACUNIT*40)
								PSO.lib.aimMissile(msl, p.pso.target)
								
								p.mo.kinsect = msl
								
								if p.mo.kinsect_stamina == nil	-- uninitialized
									p.mo.kinsect_stamina = KINSECT_MAX_STAMINA
								end
							else
							-- ungo my kinsect
								p.mo.kinsect.recall = 1
							end
						end
					end
				end,
			},

		{
			hookname = "attackSpeed",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, spd)				
						spd[1] = $+(p.mo.kinsectextract and 25 or -20)
				end,
		},

		{
			hookname = "playerThink",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, spd)				
						
						if p.mo.kinsect_stamina == nil
							p.mo.kinsect_stamina = KINSECT_MAX_STAMINA
						end
						
						if p.mo.kinsect and p.mo.kinsect.valid
							p.mo.kinsect_stamina = max(0, $-1)
							if not p.mo.kinsect_stamina
							or not p.pso.hp	-- dead
								p.mo.kinsect.recall = $ or 1
							end
						else
							p.mo.kinsect_stamina = min(KINSECT_MAX_STAMINA, $+2)
						end
						
						if p.mo.kinsectextract
							p.mo.kinsectextract = $-1
						end
				end,
		},
		
		-- nuh-huh!
		{
			hookname = "switchedWeapons",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
				p.mo.kinsectextract = nil
				if p.mo.kinsect and p.mo.kinsect.valid
					P_RemoveMobj(p.mo.kinsect)
					return
				end
			end,
		},

		{
			hookname = "switchingWeapons",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
				p.mo.kinsectextract = nil
				if p.mo.kinsect and p.mo.kinsect.valid
					P_RemoveMobj(p.mo.kinsect)
					return
				end				
			end,
		},

		{
			hookname = "playerHUD",
			type = POTTYPE_PASSIVE,
			playerargindex = 2,
			func = 	function(v, p)
				
				local bx = 272 + p.pso.palette.open*12
				local by = 134
				
				local extract = 0
				if p.mo.kinsect and p.mo.kinsect.valid
					extract = min(4, p.mo.kinsect.extract)
				end
				
				-- make it flash
				if extract == 4
				and leveltime%10 < 5
					extract = 0
				end
				
				v.draw(bx, by, v.cachePatch("H_KNSCT"..(extract+1)), V_SNAPTOBOTTOM|V_SNAPTORIGHT)
				
				v.draw(bx+32, by+2, v.cachePatch("H_KNEX"..(p.mo.kinsectextract and ((leveltime%4 < 2) or p.mo.kinsectextract > TICRATE*5) and 2 or 1)), V_SNAPTOBOTTOM|V_SNAPTORIGHT)
				
				-- kinsect stamina
				v.drawFill(bx, by+18, 45, 2, 253|V_SNAPTOBOTTOM|V_SNAPTORIGHT)
				v.drawFill(bx, by+18, (p.mo.kinsect_stamina*45/(KINSECT_MAX_STAMINA)), 1, 73|V_SNAPTOBOTTOM|V_SNAPTORIGHT)
				
			end,
		},
			
	}
}

-- Gungnir
freeslot("MT_PSO_GUNGNIR_GALE")
mobjinfo[MT_PSO_GUNGNIR_GALE] = {
	doomednum = -1,
	spawnstate = S_INVISIBLE,
	height = 32<<FRACBITS,
	radius = 16<<FRACBITS,
	flags = MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOGRAVITY
}

addHook("MobjThinker", function(mo)
	
	mo.sprite = SPR_HUWW
	mo.frame = B|FF_PAPERSPRITE|FF_TRANS50|FF_FULLBRIGHT
	mo.rollangle = $ + ANG1*25
	mo.tics = -1
	mo.color = mo.pso.combobreaker and SKINCOLOR_ORANGE or SKINCOLOR_WHITE
	mo.existtimer = $ and $+1 or 1
	local hitdelay = mo.pso.combobreaker and 15 or 30
	
	-- you don't need to see it
	if consoleplayer ~= mo.target.player
		mo.flags2 = $|MF2_DONTDRAW
	end
	
	if (mo.existtimer-1) % hitdelay == 0

		local hb = PSO.lib.spawnHurtBox(mo.x, mo.y, mo.z, 192<<FRACBITS, 255<<FRACBITS, mo.target, 15, DMT_MELEE, 2)
		hb.pso.combobreaker = mo.pso.combobreaker
	end
	
	if mo.fuse % 3 == 0
		local g = P_SpawnGhostMobj(mo)
		g.destscale = $*4
		g.rollangle = P_RandomRange(0, 359)*ANG1
	end
	
end, MT_PSO_GUNGNIR_GALE)

PSO.info.potentials["spear's chant"] = {
	
	name = "Spear's Chant",
	desc = "Normal Attack 3, Assault Buster and Raptor Throw leave gusts of wind in their wakes, hitting enemies with a Melee Potency of 15% periodically and builds or break chain depending on action used.",
	
	funcs = {


			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					if timer == 1
						p.mo.spawnedgales = 0
					end
					
					if p.pso.action == "spear_normal"
					
						if combo == 2
						and timer >= TICRATE/4
						and leveltime%5 == 0
						and (not p.mo.spawnedgales or FixedHypot(p.mo.momx, p.mo.momy) > FRACUNIT*10)
						
							local msl = PSO.lib.spawnMissile(p.mo, MT_PSO_GUNGNIR_GALE, p.drawangle+ANG1*90, 0, 1, DMT_MELEE, 0, 0, FRACUNIT*8)
							msl.fuse = TICRATE*3
							P_SetOrigin(msl, p.mo.x, p.mo.y, p.mo.z+p.mo.height/2)
							p.mo.spawnedgales = $+1
						end
					
					elseif p.pso.action == "spear_pa_assaultbuster"
					and timer > TICRATE*5
					and leveltime%3 == 0
					and (not p.mo.spawnedgales or FixedHypot(p.mo.momx, p.mo.momy) > FRACUNIT*10)
						local msl = PSO.lib.spawnMissile(p.mo, MT_PSO_GUNGNIR_GALE, p.drawangle+ANG1*90, 0, 1, DMT_MELEE, 0, 0, FRACUNIT*8)
						msl.fuse = TICRATE*4
						msl.pso.combobreaker = true
						P_SetOrigin(msl, p.mo.x, p.mo.y, p.mo.z+p.mo.height/2)	
						p.mo.spawnedgales = $+1
					
					elseif p.pso.action == "spear_pa_raptorthrow"
					and p.mo.spear and p.mo.spear.valid
					and leveltime%3 == 0
					and (not p.mo.spawnedgales or FixedHypot(p.mo.momx, p.mo.momy) > FRACUNIT*10)
						local msl = PSO.lib.spawnMissile(p.mo, MT_PSO_GUNGNIR_GALE, p.drawangle+ANG1*90, 0, 1, DMT_MELEE, 0, 0, FRACUNIT*8)
						msl.fuse = TICRATE*4
						msl.pso.combobreaker = true
						P_SetOrigin(msl, p.mo.spear.x, p.mo.spear.y, p.mo.spear.z+p.mo.height/2)
						p.mo.spawnedgales = $+1
					end
					
				end,
			},
	
	}
}

-- Soul Eater
PSO.info.potentials["soul burn"] = {
	
	name = "Soul Burn",
	desc = "Lose 1% HP every second to a minimum of 15% HP. Attack speed and critical rate increase as HP decreases up to +33% at 15% HP remaining, but reduces all healing received to 1.",
	
	funcs = {
		{
			hookname = "playerThink",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						if leveltime % TICRATE == 0
							p.pso.hp = max(1, $ - max(1, p.pso.maxhp/100))
						end
			end,
		},

		{
			hookname = "attackSpeed",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, spd)
						
						local hppercent = (p.pso.hp*100 / p.pso.maxhp)
						local nspd = min(33, (100-hppercent)*39/100)
						spd[1] = $+nspd
				end,
		},

			{
				hookname = "playerCritRate",
				type = POTTYPE_ONLYACTIVE,	-- also works for 1h weapons...
				playerargindex = 1,
				func = 	function(p, mo, inflictor, arg)
							local hppercent = (p.pso.hp*100 / p.pso.maxhp)
							local ncrit = min(33, (100-hppercent)*39/100)
							arg[1] = $+ncrit
						end,
			},

		{
			hookname = "playerHeal",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, inflictor, heal)
						
						if p.pso.hp	-- we aren't dead
							heal[1] = 1	-- we can't heal for shit!!
						end
				end,
		},
		
	},
}

-- Vijaya
PSO.info.potentials["infinite wealth"] = {
	
	name = "Infinite Wealth",
	desc = "Picking up Meseta while this weapon is equipped in any palette builds up to 5 stacks allowing the use of a Photon Art free of charge by this weapon. Normal Attack 3 will cause you to drop Meseta on the floor.",

	funcs = {

			{
				hookname = "playerPickupItem",
				type = POTTYPE_ANYPALETTE,
				playerargindex = 1,
				func = 	function(p, item, global)
					
					if type(item) == "number"
						p.mo.infinitewealth = $ and $+1 or 1
						p.mo.infinitewealth = min(5, $)
					end
				end,
			},

			{
				hookname = "PPCost",			-- hook to register into
				type = POTTYPE_ONLYACTIVE,		-- scope, when should this run?
				playerargindex = 1,				-- tells us which argument of the function the player is. (1 by default)
				func = 	function(p, cost)		-- function to run akin to the hook.
							if cost[1] > 0 
								if p.mo.infinitewealth
									cost[1] = 0
									p.mo.infinitewealth = $-1
									S_StartSound(nil, sfx_chchng)
								end
							end
						end,
			},
			
			-- drop meseta lmfao
			{
				hookname = "playerDamage",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg, skipchain, crit)
					
					if p.pso.action == "spear_normal"
					and p.pso.actioncombo == 2
						-- drop 5k * grind level meseta
						local wp = PSO.lib.getPlayerActiveWeapon(p)
						local amount = 5000*max(1, wp.grind)
						
						for i = 1, 2
							if p.pso.meseta >= amount
								
								local drop = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z+p.mo.height/2, MT_PSO_DROP)
								drop.psoitem = amount
								--print(drop)
								
								P_InstaThrust(drop, P_RandomRange(0, 359)*ANG1, P_RandomRange(4, 10)*FRACUNIT)
								drop.momz = P_RandomRange(4, 8)*FRACUNIT
								
								p.pso.meseta = $ - amount
								PSO.lib.addDropText(p, -1, "\x85-"..PSO.lib.numCommas(amount))
							end
						end
						
					end
				end,
			},			
			
			-- visual
			{
				hookname = "playerThink",
				type = POTTYPE_ANYPALETTE,
				playerargindex = 1,
				func = 	function(p)
							if p.mo.infinitewealth
								
								for i = 1, p.mo.infinitewealth
									
									local a = leveltime*ANG1*3 + FixedAngle(360*FRACUNIT/5)*i
									local sx = p.mo.x + 64*cos(a)
									local sy = p.mo.y + 64*sin(a)
									local sz = p.mo.z + p.mo.height/3
									
									local s = P_SpawnMobj(sx, sy, sz, MT_THOK)
									s.sprite = SPR_MSTA
									s.frame = ((leveltime/5) %5) |FF_FULLBRIGHT
									s.tics = 1
									
									if p ~= consoleplayer
										s.flags2 = $|MF2_DONTDRAW
									end
									
								end
							end
				end,
			},
	
	}

}

-- Ms. Confille
PSO.info.potentials["guilty sights"] = {
	
	name = "Guilty Sights",
	desc = "Revolt Aim will lock on to the current target.",
	
	funcs = {

			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					if p.pso.action == "rifle_pa_revoltaim"
					
						if p.pso.target and p.pso.target.valid and p.mo.reticle and p.mo.reticle.valid
							p.mo.reticledist = R_PointToDist2(p.mo.x, p.mo.y, p.pso.target.x, p.pso.target.y)/FRACUNIT
							P_MoveOrigin(p.mo.reticle, p.pso.target.x, p.pso.target.y, p.pso.target.z + p.pso.target.height/2)
						end
					end	
				end,
			},
	
	},
}

-- Firearms (and not Fyre Arms, you dimwits)
PSO.info.potentials["only my railgun"] = {
	
	name = "Only My Railgun",
	desc = "All projectile-based attacks become hitscan and gain a piercing attribute.",
	
	-- exceptionally handled directly in the rifle's bullet thinkers
	-- doing otherwise would've been a massive pain in the ass lol.
	funcs = {},
}

PSO.info.potentials["sub-zero barrel"] = {
	
	name = "Sub-Zero Barrel",
	desc = "Normal attacks have a 30% chance to freeze enemies that can be staggered by them, however, performing 15 normal attacks will freeze the user unless a 5s period is waited.",
	
	-- exceptionally handled directly in the rifle's bullet thinkers
	-- doing otherwise would've been a massive pain in the ass lol.
	funcs = {

			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					if p.pso.action == "rifle_normal"
					
						if timer == 9

							p.mo.frozenshooter = $ and $+1 or 1
							p.mo.frozenshootercd = TICRATE*5
							
							if p.mo.frozenshooter >= 15
								PSO.lib.inflictStatus(p.mo, COND_FREEZE, TICRATE*9)
							end
						end
					end	
				end,
			},

			{
				hookname = "playerThink",
				type = POTTYPE_ANYPALETTE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					-- timer ticks down even when weapon is sheathed
					if p.mo.frozenshootercd
						p.mo.frozenshootercd = $-1
						if not p.mo.frozenshootercd
							p.mo.frozenshooter = 0
						end
					end
				end,
			},
			
			-- freeze enemies that have 1 weight 30% of the time!
			{
				hookname = "playerDamage",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg, skipchain, crit)
				
					if mo.pso and mo.pso.weight ~= nil and mo.pso.weight <= 1
						if P_RandomRange(0, 99) < 30
							PSO.lib.inflictStatus(mo, COND_FREEZE, TICRATE*8)
						end
					end
				end,
			},

			-- weapon freeze visual
			{
				hookname = "playerThink",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					-- timer ticks down even when weapon is sheathed
					local w = p.pso.weaponpos[1][1]
					
					if p.mo.frozenshooter
						local delay = 16 - min(15, p.mo.frozenshooter)
						
						if leveltime % delay == 0
		
							local d = P_SpawnMobj(w.x + P_RandomRange(-12, 12)*w.scale, w.y + P_RandomRange(-12, 12)*w.scale, w.z + P_RandomRange(0, 12)*w.scale, MT_THOK)
							d.sprite = SPR_BARD
							d.frame = A|TR_TRANS20|FF_ADD|FF_FULLBRIGHT
							d.scale = FRACUNIT/6
							d.fuse = TICRATE/2
							d.tics = d.fuse
							d.rollangle = P_RandomRange(0, 359)*ANG1
							d.destscale = FRACUNIT/2
							d.momx = P_RandomRange(-2, 2)*w.scale
							d.momy = P_RandomRange(-2, 2)*w.scale
							d.momz = P_RandomRange(0, 2)*w.scale
							d.colorized = true
							d.color = SKINCOLOR_CYAN
						end
					end
				end,
			},
			
	},
}

-- Spread Needle
PSO.info.potentials["ancient darts"] = {
	
	name = "Ancient Darts",
	desc = "Normal attack speed +50%",
	
	funcs = {
	
			{
				hookname = "attackSpeed",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, spd)
							if p.pso.action == "shotgun_normal"
								spd[1] = $+50
							end
				end,
			},
	}
}

-- Final Impact
PSO.info.potentials["king's yield"] = {
	
	name = "King's Yield",
	desc = "On kill, gain 2% Photon Blast charge if Photon Blast is inactive, or 100% of it is active. You must be landing the killing blow to receive these effects. This potential has no effect if no Photon Blast is learned.",
	
	funcs = {
			{
				hookname = "playerDamage",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg, skipchain, crit)
				
					if PSO.lib.playerHasPB(p) and mo.pso and mo.pso.hp and dmg >= mo.pso.hp
						if PSO.lib.usingPB(p)
							p.pso.pb = PSO.info.MAXPB
						else
							p.pso.pb = $+PSO.info.MAXPB/50
						end
					end
				end,
			},		
	}
}

-- Fortress Panic
PSO.info.potentials["panic attack"] = {
	
	name = "Panic Attack",
	desc = "Hold Normal Attack to load up to 4 shots, release to let them loose at a high rate of fire. Only the initial use of Normal Attack will consume PP.",
	
	funcs = {
	
			{
				hookname = "playerActionOverride",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
							if p.pso.action == "shotgun_normal"
							and (p.pso.actiontimer == 1 or p.mo.shotgunload)
								
								local pos = p.pso.weaponpos[1]
								local potency = PSO.info.photonArts[p.pso.action].potency
								local wep = PSO.lib.getPlayerActiveWeapon(p)
								local winfo = PSO.lib.itemInfo(wep)								
								
								-- initial angles
								local angle = min(0, -90 + (timer-1)*9)
								local vangle = min(45, timer*4)
								local dist = 32
								
								local x = dist*cos(p.drawangle+(angle*ANG1))
								local y = dist*sin(p.drawangle+(angle*ANG1))
								pos[2] = x
								pos[3] = y
								pos[4] = p.mo.height/2
								pos[5] = angle*ANG1 + ANG1*90
								pos[6] = vangle*ANG1
								
								if p.pso.actiontimer == 1
									p.mo.shotgunload = 1
									p.mo.shotgunshells = 0
								end
								
								p.pso.actiontimer = min(14, $)	-- hack so we never get to "combo"
								p.pso.actionspeed = 100
								
								p.mo.shotgunload = $+1
								
								if p.mo.shotgunload >= 18 and p.mo.shotgunload <= 21
								and p.mo.shotgunshells < 4
									pos[6] = $-ANG1*10
								end
								
								if p.mo.shotgunload == 21
								and p.mo.shotgunshells < 4
									p.mo.shotgunshells = $+1
									S_StartSound(mo, p.mo.shotgunshells < 4 and sfx_pnic2 or sfx_pnic3)
									p.mo.shotgunload = 2
								end
								
								-- transition into the normal attack
								if p.mo.shotgunshells >= 1
								and not hold
									p.mo.shotgunload = nil
									p.pso.actiontimer = 2
								end
								
								return true
							end
				end,
			},

			{
				hookname = "playerActionOverride",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
							if p.pso.action == "shotgun_normal"
								
								if timer == 17
									if p.mo.shotgunshells and p.mo.shotgunshells > 1
										p.mo.shotgunshells = $-1
										
										p.pso.actiontimer = 8
										p.pso.cancancel = 0
									else
										p.mo.shotgunshells = nil
									end
								end
							end
						end,
			},
			
	}
}

-- Inferno Bazooka
PSO.info.potentials["contact ignition"] = {
	
	name = "Contact Ignition",
	desc = "On hit, inflict 8s of Burn to non-boss enemies near you. The activation distance is the same as that of the \'Zero Range\' Type Skill.",
	
	funcs = {
			-- freeze enemies that have 1 weight 30% of the time!
			{
				hookname = "playerDamage",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg, skipchain, crit)
				
					if mo.pso and not mo.pso.boss
					and R_PointToDist2(p.mo.x, p.mo.y, mo.x, mo.y) < PSO.info.CLOSERANGE*FRACUNIT
						PSO.lib.inflictStatus(mo, COND_BURN, TICRATE*8)
					end
				end,
			},
		}
}

-- SSPM Launcher
PSO.info.potentials["heatseeker"] = {
	
	name = "Heatseeker",
	desc = "Adds homing properties to projectiles if a target is locked on to. Photon Arts such as Inferno Quake and Fallen Impact will also track the target by updating their position on the fly.",
	
	-- this will also be handled in the PAs directly for convenience...
	funcs = {},
}

-- Fortress Crocket
PSO.info.potentials["pure skill"] = {
	
	name = "Pure Skill",
	desc = "Always deals 'random' critical hits.",
	
	funcs = {

			{
				hookname = "playerCritRate",
				type = POTTYPE_ONLYACTIVE,	-- also works for 1h weapons...
				playerargindex = 1,
				func = 	function(p, mo, inflictor, arg)
							arg[1] = $+999
						end,
			},
			
	}
}

-- Demonic Fork
PSO.info.potentials["devil's pierce"] = {
	
	name = "Devil's Pierce",
	desc = "Powers up piercing Techniques (Barta, Nosdiga, Ragrants, Megid, Samegid) by 25% for each hit.",
	
	funcs = {
	
			{
				hookname = "playerDamage",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg, skipchain, crit)
				
					if inflictor and inflictor.valid and inflictor.pso and inflictor.pso.pierceref
						inflictor.pso.pierceref.pso.potency = $*125/100
						--print("new pot is "..inflictor.pso.pierceref.pso.potency)
					end
				end,
			},
		}
}

-- Siren Hammer
PSO.info.potentials["siren glass"] = {
	
	name = "Siren Glass",
	desc = "Light Technique potency +15%. Non-Light Techniques have a 20% potency penalty but build up to 3 sigils allowing the use of an instant cast Light Technique. Effect remains even after switching weapons.",
	
	funcs = {
			{
				hookname = "playerTechnique",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, msl, held)
					
					local ta = p.pso.taction
					if PSO.info.photonArts[ta].element == ELEM_LIGHT
						msl.pso.potency = $*115/100
						if p.mo.sirenglass
							p.mo.sirenglass = $-1
						end
					else
						msl.pso.potency = $*8/10
						if not held
							p.mo.sirenglass = $ and $+1 or 1
							p.mo.sirenglass = min(3, $)
						end
					end
					
				end,
			},

			{
				hookname = "playerCastTime",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, tech, cast)
					
					if p.mo.sirenglass
					and tech.element == ELEM_LIGHT
						cast[1] = 1
					end
				end,
			},			

			-- visual
			{
				hookname = "playerThink",
				type = POTTYPE_ANYPALETTE,
				playerargindex = 1,
				func = 	function(p)
							if p.mo.sirenglass
								
								for i = 1, p.mo.sirenglass
									
									local a = leveltime*ANG1*3 + FixedAngle(360*FRACUNIT/3)*i + FixedAngle(360*FRACUNIT/6)
									local sx = p.mo.x + 64*cos(a)
									local sy = p.mo.y + 64*sin(a)
									local sz = p.mo.z + p.mo.height/3
									
									local s = P_SpawnMobj(sx, sy, sz, MT_THOK)
									s.sprite = SPR_SSPK
									s.frame = ((leveltime/3) %3) |FF_FULLBRIGHT
									s.tics = 1
									s.colorized = true
									s.color = SKINCOLOR_TAN
									
									if p ~= consoleplayer
										s.flags2 = $|MF2_DONTDRAW
									end
									
								end
							end
				end,
			},
			
	},
}

-- Tryshta
freeslot("MT_PSO_AFFLATUSMISERY")
mobjinfo[MT_PSO_AFFLATUSMISERY] = {
	doomednum = -1,
	spawnstate = S_INVISIBLE,
	spawnhealth = 1000,
	height = 32<<FRACBITS,
	radius = 32<<FRACBITS,
	flags = MF_NOGRAVITY|MF_NOBLOCKMAP,
}

addHook("MobjThinker", function(mo)
	
	if not mo.target or not mo.target.valid
		P_RemoveMobj(mo)
		return
	end
	
	if mo.tracer and mo.tracer.valid
		P_SetOrigin(mo, mo.tracer.x, mo.tracer.y, mo.tracer.z)
	end
	
	mo.tics = -1
	mo.fuse = -1
	mo.momx = 0
	mo.momy = 0
	mo.momz = 0
	mo.cvmem = $+1
	
	if mo.cvmem < TICRATE
		
		local scale = FRACUNIT - (mo.cvmem*FRACUNIT/30)
		
		local zpos = (TICRATE - mo.cvmem)*8*FRACUNIT
		local dist = (TICRATE - mo.cvmem)*3 + 32
		
		for i = 1, 3
			local a = leveltime*12*ANG1 + (i-1)*(360/3)*ANG1
			local x = mo.x + dist*cos(a)
			local y = mo.y + dist*sin(a)
			local s = P_SpawnMobj(x, y, zpos, MT_SUPERSPARK)
			s.colorized = true
			s.color = SKINCOLOR_RED
		end
		
		if mo.cvmem%3 == 0
			for i = 1, 64
				local s = P_SpawnMobj(mo.x, mo.y, mo.z + FRACUNIT*8*i, MT_THOK)
				s.frame = $|FF_FULLBRIGHT
				s.color = SKINCOLOR_RED
				s.scale = scale
				s.destscale = 1
				s.fuse = 11
				s.tics = 11
			end
		end
		
		if mo.cvmem == TICRATE/2
			
			S_StartSound(mo, sfx_s3k73)
			
			for i = 1, 4
				for j = 1, 16
										
					local d = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SPINDUST)
					local a = (360/16)*j*ANG1
					d.angle = a + ANG1*90
					d.renderflags = $|RF_PAPERSPRITE
										
					if i > 1
						d.spritexscale = $*2
					end
										
					d.destscale = $*4
					d.momz = (8-i)*FRACUNIT
										
					P_InstaThrust(d, a, 32*FRACUNIT*i)
					d.momx = $ + mo.momx
					d.momy = $ + mo.momy
				end
			end
		end
		return
	else
		local t = mo.cvmem - TICRATE
		
		if t == 1
			local lily = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
			lily.sprite = SPR_SIGI
			lily.frame = C|FF_FULLBRIGHT|FF_TRANS30
			lily.tics = -1
			lily.fuse = TICRATE*2
			lily.scale = FRACUNIT/2
			lily.destscale = FRACUNIT*2
			mo.lily = lily
			
			if mo.target and mo.target.player == consoleplayer
				P_StartQuake(FRACUNIT*30, 12)
				P_FlashPal(consoleplayer, 1, 4)
			end
			S_StartSound(mo, sfx_s24b)
			S_StartSound(mo, sfx_s25f)
			S_StartSound(mo, sfx_s3k66)
			
			local hb = PSO.lib.spawnHurtBox(mo.x, mo.y, mo.z, 384<<FRACBITS, 384<<FRACBITS, mo.target, mo.pso.potency, DMT_TECHNIQUE, 2)
			hb.pso.combobreaker = true
		end
		
		if leveltime % 4 == 0
		and mo.lily and mo.lily.valid
			local g = P_SpawnGhostMobj(mo.lily)
			g.colorized = true
			g.color = P_RandomRange(0,1) and SKINCOLOR_RED or SKINCOLOR_BLUE
			g.momx = P_RandomRange(-4, 4)*FRACUNIT
			g.momy = P_RandomRange(-4, 4)*FRACUNIT
			g.momz = P_RandomRange(0, 4)*FRACUNIT
			g.destscale = FRACUNIT*4
			g.scalespeed = $*2
			
			local dust = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
			dust.sprite = SPR_BARD
			dust.frame = FF_FULLBRIGHT|FF_ADD|FF_TRANS60
			dust.color = g.color
			dust.colorized = true
			dust.scale = $*3
			dust.fuse = TICRATE/3
			dust.momx = P_RandomRange(-16, 16)*FRACUNIT
			dust.momy = P_RandomRange(-16, 16)*FRACUNIT
			dust.momz = P_RandomRange(0, 4)*FRACUNIT
			
			if mo.target and mo.target.player ~= consoleplayer
				dust.flags2 = $|MF2_DONTDRAW
				g.flags2 = $|MF2_DONTDRAW
			end
		end
		
		local expanddist = 4
		local sdist = 48 + expanddist*t
		for i = 1, 4
			local a = P_RandomRange(0, 359)*ANG1
			local x = mo.x + sdist*cos(a)
			local y = mo.y + sdist*sin(a)
			
			local b = P_SpawnMobj(x, y, mo.z-FRACUNIT*256, MT_THOK)
			b.color = SKINCOLOR_WHITE
			b.frame = $|FF_FULLBRIGHT|FF_PAPERSPRITE
			b.spriteyscale = $*16
			b.spritexscale = $/4
			b.tics = -1
			b.fuse = 20
			b.momz = P_RandomRange(2, 6)*FRACUNIT
			b.angle = a+ANG1*90
			P_InstaThrust(b, a, (expanddist/2)<<FRACBITS)
			
			if mo.target and mo.target.player ~= consoleplayer
				b.flags2 = $|MF2_DONTDRAW
			end
		end
		
		if t > TICRATE*3/2
			P_RemoveMobj(mo)
		end
	end
	
	--mo.extravalue1 = $+1	
	
end, MT_PSO_AFFLATUSMISERY)

PSO.info.photonArts["afflatus misery"] = {
		-- PA is a technique
		technique = true,
		name = "Afflatus Misery",
		desc = "You should not be here.",
		cost = 0,
		actionspeed = 35,
		potency = 2200,
		icon = "H_TRFOIE",
		casttime = TICRATE*2/3,	-- how long it takes before the tech weapon swings to cast.
		
		sound = sfx_foie,
		
		reaction = RT_KNOCKBACK, -- foie does knockback
		element = ELEM_FIRE,
		spawndist = 256,		-- distance to spawn from the player if spawntarget is false
		spawntarget = true,	-- if true, spawns ontop of the target, or somewhere in front of the player if no target is detected.
		spawnheight = 50,		-- height % to spawn the object at

		object = MT_PSO_AFFLATUSMISERY,
}


PSO.info.potentials["afflatus misery"] = {
	
	name = "Afflatus Misery",
	desc = "Every 12s, gain up to 3 uses of Resta with no cast time that will also charge Afflatus Misery. Once 3 stacks are obtained, the next chain-break Technique will change into a powerful unaspected attack.",
	
	funcs = {

			{
				hookname = "playerThink",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p)
					if p.pso.pottimer and p.pso.pottimer % (TICRATE*12) == 0
						p.mo.afflatussolace = $ and $+1 or 1
						
						if p.mo.afflatussolace < 4
							S_StartSound(nil, sfx_nxitem, p)
						end
						p.mo.afflatussolace = min($, 3)
					end
				end,
			},
			
			-- consume afflatus solace
			{
				hookname = "playerCastTime",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, tech, cast)
					
					if p.mo.afflatussolace
					and p.pso.taction == "resta"
						cast[1] = 1
					end
				end,
			},	

			-- aflatus misery
			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					
					if timer == 1
					and p.pso.taction ~= "afflatus misery"
					and p.mo.afflatusmisery and p.mo.afflatusmisery >= 3
						
						-- not a support tech
						if p.pso.taction ~= "shifta"
						and p.pso.taction ~= "deband"
						and p.pso.taction ~= "jellen"
						and p.pso.taction ~= "zalure"
						and p.pso.taction ~= "diverse"
						and p.pso.taction ~= "zondeel"
						and p.pso.taction ~= "resta"
						and (p.pso.actionnum ~= 1 and p.pso.actionnum ~= 4)
							p.pso.taction = "afflatus misery"	-- that makes it easier.
							p.mo.afflatusmisery = 0
						end
					end
				end,
			},

			{
				hookname = "playerTechnique",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, msl)
					
					local ta = p.pso.taction
					if ta == "resta" and p.mo.afflatussolace
						p.mo.afflatussolace = $-1
						p.mo.afflatusmisery = $ and $+1 or 1
						p.mo.afflatusmisery = min($, 3)
					end
				end,
			},

			-- visual
			{
				hookname = "playerThink",
				type = POTTYPE_ANYPALETTE,
				playerargindex = 1,
				func = 	function(p)
							if p.mo.afflatussolace
								
								for i = 1, p.mo.afflatussolace
									
									local a = leveltime*ANG1*3 + FixedAngle(360*FRACUNIT/3)*i
									local sx = p.mo.x + 64*cos(a)
									local sy = p.mo.y + 64*sin(a)
									local sz = p.mo.z + p.mo.height/3
									
									local s = P_SpawnMobj(sx, sy, sz, MT_THOK)
									s.sprite = SPR_SSPK
									s.frame = ((leveltime/3) %3) |FF_FULLBRIGHT
									s.tics = 1
									s.colorized = true
									s.color = SKINCOLOR_CYAN
									
									if p ~= consoleplayer
										s.flags2 = $|MF2_DONTDRAW
									end
									
								end
							end

							if p.mo.afflatusmisery
								
								for i = 1, p.mo.afflatusmisery
									
									local a = leveltime*ANG1*(-3) + FixedAngle(360*FRACUNIT/3)*i
									local sx = p.mo.x + 64*cos(a)
									local sy = p.mo.y + 64*sin(a)
									local sz = p.mo.z + p.mo.height*2/3
									
									local s = P_SpawnMobj(sx, sy, sz, MT_THOK)
									s.sprite = SPR_SSPK
									s.frame = ((leveltime/3) %3) |FF_FULLBRIGHT
									s.tics = 1
									s.colorized = true
									s.color = SKINCOLOR_RED
									
									if p ~= consoleplayer
										s.flags2 = $|MF2_DONTDRAW
									end
									
								end
							end
							
				end,
			},
	
	}
}

-- Fortress Gardener
PSO.info.potentials["aerial menace"] = {
	
	name = "Aerial Menace",
	desc = "Attacks from this weapon initiated mid-air will always deal Critical Hits. Those that are not cannot deal Critical Hits.",
	
	funcs = {
	
			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					
					if timer == 1
						p.mo.marketgardener = not P_IsObjectOnGround(p.mo)
					end
				end,
			},

			{
				hookname = "playerCritRate",
				type = POTTYPE_ONLYACTIVE,	-- also works for 1h weapons...
				playerargindex = 1,
				func = 	function(p, mo, inflictor, arg)
							if p.mo.marketgardener
								arg[1] = $+999
							else
								arg[1] = $-999
							end
						end,
			},
			
	}
}

-- Axeon
PSO.info.potentials["precise shot"] = {
	
	name = "Precise Shot",
	desc = "Normal attacks shoot an instant travel projectile with a Ranged potency of 60% that builds chain.",
	
	funcs = {
	
			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					if p.pso.action == "saber_normal"	
					
						if combo == 2 and timer == TICRATE/8
						or combo < 2 and timer == TICRATE/5

							local msl = PSO.lib.spawnMissile(p.mo, MT_PSO_RIFLESHOT, p.drawangle, 90*FRACUNIT, 1, DMT_RANGED, 0, 0, FRACUNIT*8)
							msl.fuse = TICRATE
							msl.color = SKINCOLOR_YELLOW
							msl.scale = $/3
							msl.flags = $|MF_NOCLIPTHING|MF_NOBLOCKMAP|MF_NOCLIP|MF_NOCLIPHEIGHT
							S_StartSound(msl, sfx_shoot2)
							--S_StartSound(p.mo, sfx_barta)
							
							-- IF we have a target, damage it!
							if p.pso.target
								S_StartSound(p.pso.target, sfx_rail2)
								local hb = PSO.lib.spawnHurtBox(p.pso.target.x, p.pso.target.y, p.pso.target.z, 8<<FRACBITS, 8<<FRACBITS, p.mo, 60, DMT_RANGED, 2)
							end
						end
					end	
				end,
			},
	}
}

-- Elysion
PSO.info.potentials["elysion crash"] = {
	
	name = "Elysion Crash",
	desc = "Normal attack speed -30%. Normal attack 3 sends a powerful shockwave with a melee potency of 550% that knocks enemies back and builds chain.",
	
	funcs = {
	
			{
				hookname = "attackSpeed",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, spd)
							if p.pso.action == "saber_normal"
								spd[1] = $-30
							end
				end,
			},
	
			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					if p.pso.action == "saber_normal"	
					
						if combo == 2 and timer == TICRATE/8
							
							local mo = p.mo
							local pos = p.pso.weaponpos[HAND_RIGHT]
							local potency = PSO.info.photonArts[p.pso.action].potency
							local wep = PSO.lib.getPlayerActiveWeapon(p)
							local winfo = PSO.lib.itemInfo(wep)							
							local potency = 550
							local x = FixedMul(mo.scale, 100*cos(p.drawangle))
							local y = FixedMul(mo.scale, 100*sin(p.drawangle))
								
							--(x, y, z, w, t, source, potency, dtype, duration)
							local trail = PSO.lib.spawnMissile(mo, MT_SWORDEXTENDCUTTER, p.drawangle, 32*FRACUNIT, potency, DMT_STRIKE, x, y, mo.height/2)
							P_CreateFloorSpriteSlope(trail)
											
							trail.flags2 = $ | MF2_SPLAT
							trail.renderflags = $|RF_FLOORSPRITE|RF_SLOPESPLAT|RF_NOSPLATBILLBOARD
							trail.floorspriteslope.o = {x=mo.x+x, y=mo.y+y, z=mo.z+mo.height/2}
							trail.floorspriteslope.xydirection = p.drawangle
							trail.floorspriteslope.zangle = 0
							trail.rollangle = p.drawangle + ANG1*90
							trail.angle = p.drawangle
							trail.color = winfo.effectcolor or SKINCOLOR_CYAN
							trail.elysion = true
							
							P_InstaThrust(trail, p.drawangle, FRACUNIT*32)
							PSO.lib.aimMissile(trail, p.pso.target)
							trail.fuse = TICRATE

							if p == displayplayer
								P_StartQuake(FRACUNIT*10, 8)
							end

							local da = p.drawangle - ANG1*45
							for i = 1, 8
								local d = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SPINDUST)
								P_InstaThrust(d, da, FRACUNIT*P_RandomRange(46, 88))
								d.momz = P_RandomRange(0, 4)*mo.scale
								d.scale = mo.scale*2
								d.renderflags = $|RF_PAPERSPRITE
								d.angle = da + ANG1*90
								d.tics = $ + P_RandomRange(1, 6)
								d.destscale = mo.scale*3
									
								da = $ + (90/8)*ANG1
							end
						end
					end
				end,
			},
	}
}

-- Pandora Extreme
PSO.info.potentials["ex barrel"] = {
	
	name = "EX Barrel",
	desc = "Normal attack damage -85%. Each consecutive normal attack increases attack speed by 20%, up to +200%. Speed is lost when performing any other action.",
	
	funcs = {
		{
			hookname = "playerAction",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, timer, hold, input, combo)
					
				if p.pso.action == "handgun_normal"	
					if timer == 9
						p.mo.exbarrel = $ and $+1 or 1
					end
				else
					p.mo.exbarrel = nil
				end
			end,
		},

		{
			hookname = "attackSpeed",
			type = POTTYPE_ONLYACTIVE,
			playerargindex = 1,
			func = 	function(p, spd)
						if p.pso.action == "handgun_normal"
							spd[1] = $+min(200, 20*(p.mo.exbarrel or 0))
						end
			end,
		},	

		{
			hookname = "switchedWeapons",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
				p.mo.exbarrel = nil	-- this is a hack.
			end,
		},

		{
			hookname = "switchingWeapons",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
				p.mo.exbarrel = nil	-- this is a hack.
			end,
		},

		{
			hookname = "playerDamageCalc",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			inflictorindex = 3,
			func = 	function(p, mo, inflictor, dmg)
				
				if inflictor.type == MT_PSO_RIFLESHOT
				and not inflictor.pso.combobreaker
				and inflictor.fuse
					dmg[1] = $*15/100
				end
			end,
		},
	}
}

-- Ruby Bullet
PSO.info.potentials["scorching round"] = {
	
	name = "Scorching Round",
	desc = "Attacks inflict a stacking 3s Burn onto non-boss enemies, up to 30s.",
	
	funcs = {
	
			{
				hookname = "playerDamage",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, dmg, skipchain, crit)
				
					if mo.pso and not mo.pso.boss
					and PSO.lib.getPlayerActiveWeapon(p).weapontype == WPT_HANDGUN
						local time = TICRATE*3
						if mo.pso.condition == COND_BURN
							mo.pso.conditiontimer = min(TICRATE*30, $ + time)
						else
							PSO.lib.inflictStatus(mo, COND_BURN, time)
						end
					end
				end,
			},
	}
}

-- Evoker...
freeslot("MT_PSO_ORPHEUS", "SPR_PSN0", "sfx_qsumon", "sfx_arm", "sfx_summon", "sfx_1more", "sfx_pcrit")

mobjinfo[MT_PSO_ORPHEUS] = {
	doomednum = -1,
	spawnstate = S_INVISIBLE,
	spawnhealth = 1000,
	height = 32<<FRACBITS,
	radius = 32<<FRACBITS,
	flags = MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOGRAVITY,
}	

local orpheus_anim_attack = {D, E, F, G, H, H, H, I, J, K, L, L, L, L, L, L}
local orpheus_anim_stand = {A, B, C, B}

addHook("MobjThinker", function(mo)
	
	mo.sprite = SPR_PSN0
	mo.colorized = true
	mo.color = SKINCOLOR_TEAL
	
	if mo.spawntimer
		mo.frame = D
		mo.spriteyscale = mo.spawntimer*FRACUNIT
		mo.spritexscale = FRACUNIT/mo.spawntimer
		mo.spawntimer = $-1
	
	elseif mo.animtimer
		mo.animtimer = $+1
		
		local frame = orpheus_anim_attack[1 + mo.animtimer/2]
		if not frame
			mo.animtimer = nil
		else
			mo.frame = frame
		end
	else	-- stand anim
		
		mo.despawntimer = $ and $+1 or 1
		mo.spriteyscale = mo.despawntimer*FRACUNIT
		mo.spritexscale = FRACUNIT/mo.despawntimer
		
		if mo.despawntimer > 3
			P_RemoveMobj(mo)
			return
		end
	end
	
	mo.frame = $|FF_FULLBRIGHT|FF_ADD|FF_TRANS40
	
end, MT_PSO_ORPHEUS)

PSO.info.potentials["1more"] = {
	
	name = "1More!",
	desc = "Technique damage will scale off of R-ATK. Detonating Chain with a Technique that targets a weakness will restart the Chain at a quarter of the count after Chain Break expires.",
	
	funcs = {

			{
				hookname = "playerCastTime",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, tech, cast)
					p.mo.evokercast = cast[1]	-- remember this for our anim hack, teehee...
				end,
			},	

			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					local mo = p.mo
					local pos = p.pso.weaponpos[1]
					
					if p.pso.action ~= "rod_cast" return end	-- bruh
					
					local casttime = p.mo.evokercast
					if casttime == nil return end
					
					mo.evokertime = $ and $+1 or 1
					
					if timer == 1
						p.mo.evokertime = 1
						--S_StartSound(mo, sfx_qsumon)
					end
					
					local a = p.drawangle - ANG1*95	-- for layering
					local z = mo.height/3 + 48*sin(min(90+73, mo.evokertime*13)*ANG1)
					local dist = 16 + (z/3)/FRACUNIT
					local x = dist*cos(a)
					local y = dist*sin(a)
					
					if timer >= casttime
					and mo.evokertime < 13
						mo.evokertime = 13
					end
					
					if mo.evokertime == 13
						S_StartSound(mo, sfx_arm)
					end
					
					pos[2] = x
					pos[3] = y
					pos[4] = z
					pos[5] = ANG1*90
					pos[6] = mo.evokertime <= 10 and mo.evokertime*ANG1*30 or 0
					pos[7] = {use=false}
					
					--print(timer.."/"..casttime)
					
					if timer == casttime-1
						for i = 1, 8
							local part = P_SpawnMobj(pos[1].x, pos[1].y, pos[1].z, MT_SUPERSPARK)
							part.color = SKINCOLOR_TEAL
							part.colorized = true
							part.angle = p.drawangle + ANG1*90
							P_InstaThrust(part, part.angle, P_RandomRange(16, 32)*FRACUNIT)
							part.momz = P_RandomRange(-5, 5)*FRACUNIT
							part.scale = $/2
						end
						--P_FlashPal(p, 1, 4)
						S_StartSound(mo, sfx_summon)
						
						-- spawn orpheus behind the player
						local sx = mo.x + 32*cos(p.drawangle+ANG1*180)
						local sy = mo.y + 32*sin(p.drawangle+ANG1*180)
						local sz = mo.z + mo.height*3/4
						
						local persona = P_SpawnMobj(sx,sy,sz, MT_PSO_ORPHEUS)
						persona.target = mo
						persona.angle = p.drawangle
						persona.animtimer = 1
						persona.spawntimer = 4
						mo.persona = persona
					end
				end,
			},
			
			-- the actual ability...
			{
				hookname = "playerChainBreak",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor, damage, firstchain)
							
							if not firstchain
							and inflictor and inflictor.valid
							and inflictor.pso
							and inflictor.pso.element
							and mo.pso
							and (mo.pso.weakness == inflictor.pso.element or mo.pso.weakness2 == inflictor.pso.element)	
								mo.pso.chainrestart = (mo.pso.chain+1)/4 or 1
								S_StartSound(mo, sfx_pcrit)
							end
				end,
			},
			
			-- make techniques spawn on orpheus
			{
				hookname = "playerTechnique",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, msl)
					
					local ta = p.pso.taction
					local tech = PSO.info.photonArts[ta]
					if not tech.spawntarget
					and p.mo.persona and p.mo.persona.valid	-- sanity checks

						local dist = tech.spawndist or 0
						local offx = dist*cos(p.drawangle)
						local offy = dist*sin(p.drawangle)
						local offz = FixedDiv(48*FRACUNIT*(tech.spawnheight or 0)/100, p.mo.scale)
							
						local spd = tech.speed
						if spd == nil
							spd = 40	-- 0 spd is still valid
						end
							
						P_SetOrigin(msl, p.mo.persona.x+offx, p.mo.persona.y+offy, p.mo.persona.z+offz) 
						if tech.aim
							PSO.lib.aimMissile(msl, p.pso.target)
						end
					end
				end,
			},			
			
			-- make techs scale off of r-atk (this is cursed)
			{
				hookname = "playerPreDamageCalc",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = function(p, mo, inflictor)
					
					if inflictor and inflictor.valid and inflictor.pso and inflictor.pso.damagetype == DMT_TECHNIQUE
						inflictor.pso.damagetype = DMT_RANGED
					end
				end,
			},
			
			-- check server chain trackers continuously to apply 1more if possible
			{
				hookname = "playerThink",
				type = POTTYPE_ANYPALETTE,
				playerargindex = 1,
				func = function(p)
					if server.chaintrackers
						
						for i = 1, #server.chaintrackers
							
							local mo = server.chaintrackers[i]
							local info = mo.pso
							
							if info.chainbreak == 1
							and info.chainrestart
								info.chain = max(2, info.chainrestart)
								info.chainbreak = nil
								info.chaintime = max(TICRATE/2, TICRATE*5 - (info.chain-1)*3/2)
								
								info.chainrestart = nil	-- take this away
								
								local a = R_PointToAngle(mo.x, mo.y)
								local x = mo.x - 32*cos(a)
								local y = mo.y - 32*sin(a)
								local more = P_SpawnMobj(x, y, mo.z + FRACUNIT*64, MT_THOK)
								more.sprite = SPR_SIGI
								more.frame = D|FF_FULLBRIGHT|FF_ADD
								more.color = SKINCOLOR_TEAL
								more.tics = -1
								more.fuse = TICRATE
								more.scale = 1
								more.destscale = FRACUNIT
								more.scalespeed = FRACUNIT/5
								
								S_StartSound(mo, sfx_1more)
							end
						end
					end
				end,
			},
			
	},
}

-- Rykros Staff
PSO.info.potentials["eye of rykros"] = {
	
	name = "Eye of Rykros",
	desc = "While under the effect of a Photon Blast, casting any Technique other than Megid or Samegid will additionally cast Megid, building or breaking Chain to match the Technique cast.",
	
	funcs = {

			{
				hookname = "playerTechnique",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, msl)
					
					local mo = p.mo
					local ta = p.pso.taction
					if ta ~= "megid" and ta ~= "samegid" and PSO.lib.usingPB(p)
						-- cast megid11!!!
						local tech = PSO.info.photonArts["megid"]
						local spd = (tech.speed or 40)*FRACUNIT
						local pot = tech.potency
						local wep = PSO.lib.getPlayerActiveWeapon(p)
						local dist = tech.spawndist or 0
						local offx = dist*cos(p.drawangle)
						local offy = dist*sin(p.drawangle)
						local offz = FixedDiv(48*FRACUNIT*(tech.spawnheight or 0)/100, mo.scale)
						
						local msl2 = PSO.lib.spawnMissile(mo, MT_PSO_MEGID, p.drawangle, spd, pot, DMT_RANGED, offx, offy, offz)
						msl2.pso.combobreaker = msl.pso.combobreaker	-- charge shots are always combo breakers

						if tech.sound
							S_StartSound(mo, tech.sound)
						end
							
						if tech.element
							msl2.pso.element = tech.element
						end
							
						local range = wep.range or 512
						if range and spd
							msl2.fuse = FixedDiv(range*FRACUNIT, max(1, FixedHypot(msl.momx, msl.momy)))/FRACUNIT
						end
							
						PSO.lib.aimMissile(msl2, p.pso.target)
					end
				end,
			},
	},
}

-- Evil Curst
PSO.info.potentials["befallen curse"] = {
	
	name = "Befallen Curse",
	desc = "Dark element Techniques apply a stacking 5s of Jellen and Zalure.",
	
	funcs = {

			{
				hookname = "playerDamage",
				type = POTTYPE_PASSIVE,
				playerargindex = 1,
				func = 	function(p, mo, inflictor)
					
					if inflictor and inflictor.pso.damagetype == DMT_TECHNIQUE and inflictor.pso.element == ELEM_DARK
						if mo.pso
							PSO.lib.addJellen(mo, TICRATE*5)
							PSO.lib.addZalure(mo, TICRATE*5)
						end
					end
				end,
			},
	},
}

-- Crimson Demon Staff
PSO.info.potentials["explosion"] = {
	
	name = "Explosion",
	desc = "Increases Rafoie's cast time by 400% and its potency and radius by 400%. Additionally, each succesful cast will cost 20% of the user's HP which may result in incapacitation as well as Mate Sickness.",
	
	funcs = {

			{
				hookname = "playerCastTime",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, tech, cast)
					
					if p.pso.taction == "rafoie"
					and p.pso.action == "wand_cast"
						cast[1] = $*5
						p.mo.rafoiecast = cast[1]
					end
				end,
			},

			-- ECKSUPLOOOOOOOOOSION
			{
				hookname = "playerTechnique",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, msl)
					
					if p.pso.taction == "rafoie"
					and p.pso.action == "wand_cast"
						
						PSO.lib.damageObject(p.mo, nil, p.pso.maxhp*2/10)
						p.pso.matesickness = $ and $+ TICRATE*20 or TICRATE*20
						
						local origin = {x=msl.x, y=msl.y, z=msl.z}
						if p.pso.target and p.pso.target.valid
							origin = {x=p.pso.target.x, y=p.pso.target.y, z=p.pso.target.z}
						end
						
						local hb = PSO.lib.spawnHurtBox(origin.x, origin.y, origin.z - msl.scale*32, 650<<FRACBITS, 512<<FRACBITS, p.mo, msl.pso.potency*5, DMT_TECHNIQUE, 8, RT_BLOWDOWN)
						hb.pso.combobreaker = msl.pso.combobreaker
						hb.pso.reactionradialangle = true
						
						-- I'll show you what an explosion's like!!
						for i = 1, 4
							-- spawn rings
							for j = 1, 16
								local a = FixedAngle(FRACUNIT*360 / 16)*j
								local e = P_SpawnMobj(origin.x, origin.y, origin.z + (i-1)*192*FRACUNIT, MT_SONIC3KBOSSEXPLODE)
								e.scale = P_RandomRange(4, 8)*FRACUNIT/2
								e.angle = a+ANG1*90
								e.renderflags = $|RF_PAPERSPRITE|RF_FULLBRIGHT
								P_InstaThrust(e, a, FRACUNIT*30 - (FRACUNIT*5*(i-1)))
								
								local dust = P_SpawnMobj(e.x, e.y, e.z, MT_TNTDUST)
								dust.momx = e.momx/4
								dust.momy = e.momy/4
								dust.scale = $*3
							end
						end
						
						for i = 1, 64
							local x = origin.x + P_RandomRange(-96, 96)*FRACUNIT
							local y = origin.y + P_RandomRange(-96, 96)*FRACUNIT
							local z = origin.z + P_RandomRange(-96, 96)*FRACUNIT
							
							local e = P_SpawnMobj(x, y, z, MT_SONIC3KBOSSEXPLODE)
							e.spriteyscale = P_RandomRange(1, 4)*FRACUNIT
							e.spritexscale = FRACUNIT/2
							e.momz = e.spriteyscale
							e.scale = $*2
							e.tics = $ + P_RandomRange(0, 8)
						end
						
						if p == consoleplayer
							P_FlashPal(p, 1, 3)
							P_StartQuake(FRACUNIT*50, 10)
						end
						
						local sound = P_SpawnMobj(origin.x, origin.y, origin.z, MT_THOK)
						sound.state = S_INVISIBLE
						sound.tics = TICRATE*8
						S_StartSound(sound, sfx_bgxpld)
						
						P_RemoveMobj(msl)
					end
				end,
			},
			
			{
				hookname = "playerAction",
				type = POTTYPE_ONLYACTIVE,
				playerargindex = 1,
				func = 	function(p, timer, hold, input, combo)
					
					if p.pso.action == "wand_cast"
					and p.pso.taction == "rafoie"
					and p.mo.rafoiecast
					
						local mo = p.mo
						local pos = p.pso.weaponpos[1]
						
						if timer < p.mo.rafoiecast
							
							local x = mo.x
							local y = mo.y
							local z = mo.z+mo.height + mo.scale*56
							
							local test = P_SpawnMobj(x, y, z, MT_THOK)
							test.sprite = SPR_SIGI
							test.frame = E|FF_TRANS50|FF_FULLBRIGHT
							
							test.tics = 1
							test.rollangle = -leveltime*ANG1*7
							
							if leveltime%3 == 1
								local g = P_SpawnGhostMobj(test)
								g.destscale = 1
								g.tics = $*2
							end
							
							local s = P_SpawnMobj(x, y, z, MT_THOK)
							s.sprite = SPR_SIGI
							s.frame = G|FF_FULLBRIGHT|FF_ADD
							s.tics = 1
							
							if timer == 1
								local g = P_SpawnGhostMobj(s)
								g.destscale = $*4
								g.scalespeed = $*4
								g.tics = $*4
							end
							
							s.scale = min(FRACUNIT, timer*FRACUNIT/6)
						end
					end
				end,
			},
		
	}
}

-- Warrior's weapons
PSO.info.potentials["dynamo charge"] = {
	
	name = "Dynamo Charge",
	desc = "Attack speed +35% for 20s after evading or blocking an attack. Effect is lost upon switching weapons. If the weapon is 1-handed, both weapons are affected.",

	funcs = {
	
		{
			hookname = "playerDodge",
			type = POTTYPE_PASSIVE,	-- also works for 1h weapons...
			playerargindex = 1,
			func = 	function(p)
						S_StartSoundAtVolume(p.mo, sfx_cburst, 127)
									
						for i = 1, 8
							local s = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_SUPERSPARK)
							s.destscale = 1
							s.momx = P_RandomRange(-32, 32)*FRACUNIT
							s.momy = P_RandomRange(-32, 32)*FRACUNIT
							s.momz = P_RandomRange(-32, 32)*FRACUNIT
							s.colorized = true
							s.color = SKINCOLOR_CYAN
						end
						
						p.mo.warrior_speed = TICRATE*20
					end,
		},
		
		{
			hookname = "playerThink",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						if p.mo.warrior_speed
							p.mo.warrior_speed = $-1
							
							if leveltime %2
								for i = 1, P_RandomRange(1, 4)
									local a = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z +p.mo.height/4, MT_THOK)
									a.sprite = SPR_KLIT
									a.colorized = true
									a.color = SKINCOLOR_CYAN
									a.scale = p.mo.scale/6
									a.tics = 4
									a.frame = P_RandomRange(0, P) |FF_FULLBRIGHT|TR_TRANS40|FF_ADD
									a.momx, a.momy, a.momz = p.mo.momx, p.mo.momy, p.mo.momz
								end								
							end
						end
					end,
		},

		{
			hookname = "attackSpeed",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p, spd)
						if p.mo.warrior_speed
							spd[1] = $+35
						end
				end,
		},

		{
			hookname = "switchedWeapons",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						p.mo.warrior_speed = nil
					end,
		},
	},
	
}

-- TCSM/ Angel
PSO.info.potentials["beats from beyond"] = {
	
	name = "Beats from Beyond",
	desc = "Revives the user at full HP and PP once per quest if equipped at the time of incapacitation.",
	
	funcs = {

		{
			hookname = "playerThink",
			type = POTTYPE_PASSIVE,
			playerargindex = 1,
			func = 	function(p)
						
						if p.pso.deadtimer == 3
						and not p.mo.kanade_raise
							p.mo.kanade_raise = true
							p.mo.kanade_raisetime = 1
							p.pso.hp = 1
							--print("Begin raise?")
						end
						
						if p.mo.kanade_raisetime
							p.mo.kanade_raisetime = $+1
							p.mo.noactiontimer = 1	-- specific tech-mag hack that exploits the delay before they return
							p.pso.deadtimer = min(TICRATE*3/4, $)
							
							-- move kanadium around the player
							local pos = p.pso.weaponpos[1]	-- right hand
							
							local a = leveltime*ANG1*(-12)
							local z = p.mo.kanade_raisetime * FRACUNIT/2
							
							pos[2] = 48*cos(a)
							pos[3] = 48*sin(a)
							pos[4] = z
							pos[5] = R_PointToAngle2(0, 0, pos[1].momx, pos[1].momy) - p.drawangle
							
							if leveltime%5 == 0
								P_SpawnMobj(pos[1].x, pos[1].y, pos[1].z, MT_SUPERSPARK)
							end
							
							if p.mo.kanade_raisetime > TICRATE
								p.pso.hp = min(p.pso.maxhp, $+p.pso.maxhp/70)
								p.pso.pp = min(p.pso.maxpp, $+p.pso.maxpp/70)
								
								p.mo.momz = FRACUNIT/2
								
								if p.mo.kanade_raisetime == TICRATE*3
									PSO.lib.revivePlayer(p, p, p.pso.maxhp)
									p.pso.pp = p.pso.maxpp
									p.mo.state = S_PLAY_SUPER_TRANS1
									p.mo.kanade_raisetime = nil
								end
							end
						end
					end,
		},
	
	}
}

-- register all our potentials...
for k,v in pairs(PSO.info.potentials)
	PSO.lib.registerPotential(k)
end