-- Builds data relative to level synch such as the maximum atk achievable by each weapon type at any given level...

/*
	How does level synch work?
	
	-> Player base stats are set to the synched level's stats
	-> Player's equipped weapon's attack stat cannot exceed the value of the highest attack achievable with a +10 weapon at that level
	-> Player's equipped armour's def stat cannot exceed the value of the highest defence achievanle with a +10 armour at that level
	-> Units are handled differently:
	
		-> Unit stat buffs and penalties are reduced by a factor based on:
			-> Maximum unit tier achievable, following this conversion table:
			(penalties to 1* conversion are harsher by design.)
			
			From 10*
			To 7*: 0.7x
			To 4*: 0.45x
			To 1*: 0.20x
			
			From 7*
			To 4*: 0.6x
			To 1*: 0.3x
			
			From 4*
			To 1*: 0.5x
		
		-> Further dampening is applied if you have more units equipped than normally available number of slots.
			-> This scalar will always be <synched slots> / <current slots>
		
		-> Stat dampening cannot cause a stat to reach 0, it will always be 1 or -1 at minimum.
		
		-> The following stats are *NOT* affected by scaling on units:
			-> PP Regen Speed
			-> Attack Speed
			-> Critical Rate
			-> Critical Damage
*/

PSO.info.levelSynchData = {}
local synch = PSO.info.levelSynchData

local weapontypes = {
	WPT_SWORD, WPT_DOUBLESABER, WPT_SPEAR, WPT_SABER, WPT_RIFLE, WPT_SHOTGUN, WPT_LAUNCHER, WPT_HANDGUN, WPT_ROD, WPT_WAND, WPT_TECHMAG
}

synch.weaponBaseAttack = {}
synch.weaponMaxAttack = {}
for i = 1, #weapontypes
	local wtype = weapontypes[i]
	synch.weaponMaxAttack[wtype] = {}
	synch.weaponBaseAttack[wtype] = {}
	
	for lv = 1, PSO.info.LEVELCAP
		-- find the weapon of the given weapon type with the highest atk equippable at this level
		local highest_atk = 0
		local highest_batk = 0
		
		for k,v in pairs(PSO.info.items)
			if v.weapontype and v.weapontype == wtype
			and (not v.req or v.req <= lv)
				
				-- grind the weapon to +10 implicitely.
				-- weapons ground to +20 or above will always be gated behind higher levels.
				
				local batk = v.atk
				local atk = batk + 10*(1 + (v.rarity-1)/3)
				highest_batk = max($, batk)
				highest_atk = max($, atk)
				
			end
		end
		
		synch.weaponBaseAttack[wtype][lv] = highest_batk
		synch.weaponMaxAttack[wtype][lv] = highest_atk
		--print("T: "..wtype..", Lv"..lv..": "..highest_atk.."ATK")
	end
end

-- returns the atk stat for a synched weapon.
PSO.lib.lvSynchWeapon = function(p, level, weapon)
	
	if p.pso.level < level
		return
	else
		
		-- low-level synch
		/*if p.pso.levelsynch_low
			local baseatk = synch.weaponBaseAttack[weapon.weapontype][level]
			if weapon.atk >= baseatk
				return
			else
				return baseatk
			end
			return
		end*/
		
		local grind
		
		if p.pso.levelsynch_grind ~= nil
			grind = p.pso.levelsynch_grind
		end
		
		if grind == nil
			grind = 10
		end
		
		-- do nothing if there is no entry for that level (synch above cap???) or the weapon is weaker than max atk.
		local baseatk = synch.weaponBaseAttack[weapon.weapontype][level]
		local maxatk = synch.weaponMaxAttack[weapon.weapontype][level]
		
		if not maxatk return end
		
		local atkdiff = (maxatk-baseatk)/PSO.info.MAXGRIND
		
		local synchatk = baseatk + atkdiff*grind
		
		if weapon.atk <= synchatk
			return
		end
		
		return synchatk
	end
end

synch.armourBaseDefence = {}
synch.armourMaxDefence = {}
for lv = 1, PSO.info.LEVELCAP
	
	local max_def = 0
	local max_bdef = 0
	
	for k,v in pairs(PSO.info.items)
		if v.type == IT_ARMOR
		and (not v.req or v.req <= lv)
			local def = v.def + 10*(1 + (v.rarity-1)/3)
			max_bdef = max($, v.def)
			max_def = max($, def)
		end
	end
	
	synch.armourBaseDefence[lv] = max_bdef
	synch.armourMaxDefence[lv] = max_def
	--print("Lv"..lv..": "..max_slots.." slots")
end

-- max unit rarity available at this level
-- this is also used to know much to cut unit stats by
synch.maxUnitTier = {}
for lv = 1, PSO.info.LEVELCAP

	local max_tier = 0
	
	for k,v in pairs(PSO.info.items)
		if v.type == IT_ARMOR
		and (not v.req or v.req <= lv)
			max_tier = max($, v.rarity)
		end
	end
	
	local extra = (max_tier-1)%3
	max_tier = $-extra
	
	synch.maxUnitTier[lv] = max_tier
	--print("Lv"..lv..": "..max_tier.."*")
end

PSO.lib.lvSynchArmour = function(p, level, armour)
	
	if p.pso.level < level
		return
	else

		-- low-level synch
		/*if p.pso.levelsynch_low
			local basedef = synch.armourBaseDefence[level]
			if armour.def >= basedef
				return
			else
				return basedef
			end
			return
		end*/

		local grind
		
		if p.pso.levelsynch_grind ~= nil
			grind = p.pso.levelsynch_grind
		end
		
		if grind == nil
			grind = 10
		end
		
		-- do nothing if there is no entry for that level (synch above cap???) or the armour is weaker than max def.
		local maxdef = synch.armourMaxDefence[level]
		local basedef = synch.armourBaseDefence[level]
		
		if not maxdef return end
		local diff = (maxdef-basedef)/PSO.info.MAXGRIND
		
		local synchdef = basedef + diff*grind
		
		local ainfo = PSO.lib.itemInfo(armour)
		local synchhp = (ainfo.hp or 0) * synch.maxUnitTier[level] / armour.rarity
		local synchpp = (ainfo.pp or 0) * synch.maxUnitTier[level] / armour.rarity
		
		if armour.def <= synchdef
			return nil, synchhp, synchpp
		end
		
		return synchdef, synchhp, synchpp
	end
end


-- highest # of unit slot per level
-- this is used to know how much to cut unit stats by
synch.maxUnitSlots = {}
for lv = 1, PSO.info.LEVELCAP
	
	local max_slots = 0
	
	for k,v in pairs(PSO.info.items)
		if v.type == IT_ARMOR
		and (not v.req or v.req <= lv)
			max_slots = max($, v.slots or 0)
		end
	end
	
	synch.maxUnitSlots[lv] = max_slots
	--print("Lv"..lv..": "..max_slots.." slots")
end

local unit_conversion_dampening = {
	[10] = {
		[7] = FRACUNIT*7/10,
		[4] = FRACUNIT*45/100,
		[1] = FRACUNIT*2/10,
	},
	
	[7] = {
		[4] = FRACUNIT*6/10,
		[1] = FRACUNIT*3/10,
	},
	
	[4] = {
		[1] = FRACUNIT/2,
	}
}

-- okAY THIS IS GOING TO FUCKING SUCK
PSO.lib.lvSynchUnit = function(p, level, unit, currentslots)
	
	
	if p.pso.level < level
	or p.pso.levelsynch_low	-- nope.
	--or currentslots == 0	-- #div 0
	or (not unit.hp and not unit.pp and not unit.satk and not unit.ratk and not unit.tatk)	-- no relevant stats
		--print("lv ok / no slots")
		return
	end
	
	local maxslots = synch.maxUnitSlots[level]
	
	local dampen = FRACUNIT	-- = 1.0x
	
	local rarity = unit.rarity
	-- dumb this down:
	local extra = (rarity-1)%3
	rarity = $-extra
	
	local max_rarity = synch.maxUnitTier[level]
	
	if not max_rarity
		--print("no max tier")
		return	-- what
	end
	
	-- there is no reason to dampen it.
	if unit.rarity <= max_rarity
	and maxslots >= currentslots
		return
	end
	
	-- apply dampening.
	if rarity > max_rarity
		dampen = FixedMul($, unit_conversion_dampening[rarity][max_rarity] or FRACUNIT)
	end
	
	-- further dampen based on current slots / max slots.
	if currentslots > maxslots
		local fac = FixedDiv(maxslots<<FRACBITS, currentslots<<FRACBITS)
		dampen = FixedMul($, fac)
	end
	
	-- finally, apply this to all the stats...
	-- other stats are not affected.
	local stats = {"hp", "pp", "satk", "ratk", "tatk"}
	local retval = {}
	
	for i = 1, #stats
		local stat = stats[i]
		
		retval[stat] = (FixedMul((unit[stat] or 0)*FRACUNIT, dampen) / FRACUNIT)
		if unit[stat]	-- stat cannot go below 1 or -1 depending on if it is positive or negative, but EXISTS.
			if unit[stat] > 0
				retval[stat] = $ or 1
			else
				retval[stat] = $ or -1
			end
		end
	end
	
	return retval
end